###############################################################_
#---BIBLIOTHEK - Funktionen ----
source(file = "../FUNKTIONEN.R")

#---Lokale Funktionen ----
NA_in_0     = function(x)      {POS_NA = which(is.na(x)==TRUE); x[POS_NA] = 0; return(x)} #Umwandlung von NA in 0
NA_all      = function(x,Spalte)      {if(is.vector(x)){NA_TRUE=sum(is.na(x));length(x)==NA_TRUE}else{NA_TRUE=sum(is.na(x[,Spalte])); length(x[,Spalte])==NA_TRUE}}
euc.dist    = function(x1, x2) {sqrt(sum((x1 - x2) ^ 2))} #Berechnung der euklidischen Distanz 
is.integer0 = function(x)      {is.integer(x) && length(x) == 0L} #Pruefung ob Variable ein integer ist TRUE/FALSE 
delete.na   = function(DF, n=0) {DF[rowSums(is.na(DF)) <= n,]} #Loeschen von Zeilen mit definierter Anzahl an NA´s 

###############################################################_
#-#-#-# BIBLIOTHEK - package  ----
library(ISI.Toolbox);library(zoo);library(dygraphs);library(MASS)
library(matrixStats);library(ggplot2);library(scales)
library(Metrics);library(wesanderson);library(gridExtra);library(grid)
library(XLConnect)

###############################################################_
#---LDO-Rohdaten einlesen  ----
LinkPridatenbank    = "../../1_Daten/2_Optimierter Kohlenstoffabbau - Labor/1_Primaerdaten/"
Versuchstage = c("2013-03-11",  "2013-04-02", "2013-04-09", "2014-04-08", 
                 "2014-04-28", "2014-05-05", "2018-02-13", "2018-03-05", "2019-02-21",
                 "2019-02-25", "2019-02-26", "2019-03-05", "2019-04-08", "2019-04-10",
                 "2020-03-05")
#Tag=4
NO3_N = c(TRUE,TRUE,TRUE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE)

for (Tag in 4:length(Versuchstage)) {
  print(paste("Zeitreihe von Versuchstag: ",Versuchstage[Tag], " ausgewählt"))
  O2Versuchstag = dir(paste(LinkPridatenbank, Versuchstage[Tag], "/O2/", sep=""))
  
  O2Data        = read.table(file = paste(LinkPridatenbank, Versuchstage[Tag], "/O2/",Versuchstage[Tag], "_O2Konzentration.txt", sep=""),
                             sep=" ", header=T, dec = ".", skip = 1)
  if(ncol(O2Data)==3){datime = O2Data[,1]}else{datime = paste(O2Data[,1], O2Data[,2],sep = " ")} #WTW oder LDO Dataframe
  
  Datime = strptime(datime, format="%Y-%m-%d %H:%M:%S", tz="UTC")
  if(ncol(O2Data)==3){DF = data.frame(Datime, O2Data[,-1])}else{DF = data.frame(Datime, O2Data[,-1:-2])} #WTW oder LDO Dataframe
  
  colnames(DF) = c("Datime", "O2", "Temperatur")
  
  minuten = difftime(DF$Datime,DF$Datime[1],units='mins')
  stunden = round(minuten/60, digits = 0)
  
  ##################################################################_
  ##################################################################_
  #----PLOT: Sauerstoffkonzentration ----
  if(max(stunden)>25){
    x_aufteilung = stunden*60*10
    x_beschriftung = stunden*10
    plot_height = 7; plot_width = 21
  }else{
    x_aufteilung = stunden*60*2
    x_beschriftung = stunden*2
    plot_height = 7.5; plot_width = 10}
  
  pdf(file = paste("2_Abbildungen/1_Sauerstoffkonzentation/", Versuchstage[Tag] ,".pdf", sep=""),  width = plot_width, height = plot_height)#, width = 300, height = 400)
  par(mar=c(5,6,4,2)+0.1, oma = c(0, 0, 0, 2))
  plot(minuten, DF$O2, type = "l", ylab = expression("Konzentration [mg "* L^-1*"]" ),lwd=2, xlab="Versuchszeit [h]",
       cex.axis=2, xaxt = "n", cex.lab=2)
  grid(nx = NA, ny = NULL, lwd = 2, col="gray50", lty = 2)
  abline(v = x_aufteilung , col="gray50", lty=2, lwd =2)
  lines(minuten, DF$O2,lwd=2); box()
  axis(1,  at=x_aufteilung , labels=x_beschriftung,  cex.axis=2, las=1, mgp=c(3, 1.3, 0))
  legend("topright",cex=2, y=NULL, legend = c("Sauerstoff"), inset = 0.05, col = c("black"), lty=c(1),
         lwd = c(2), bg = alpha("white", 0.6))
  dev.off()
  # ##################################################################_
  # ##################################################################_
  dt  = as.vector(diff(minuten))     #---Zeitdifferenz
  do2 = diff(DF$O2)                  #---O2-Differenz zwischen Minutenwerten
  
  # ###############################################################_
  # #---INFO: Index, um Start- und Endwerte der Belueftung herauszusuchen
  index_zehr=which(do2<0) # "1" damit erster Wert auch eingebunden wird und ?ueberprueft werden kann?!
  
  index_belueft=which(do2>=0)
  index_belueft=c(2,index_belueft)                    # "2" im Fallbeispiel steht 2 vor 1 und damit eine Differenz von -1 und damit kleiner als 1
  #
  i_starte=index_zehr[which(diff(index_zehr)>1)+1]      # i_starte & i_ende als Positionen im Zeitvektor (Minuten)
  i_ende=index_belueft[which(diff(index_belueft)>1)+1]
  
  #---INFO: Berechnung der Steigungskoeffizienten der Zehrungsintervalle
  i=1; tour=NULL; coef.our=NULL; dt.zehr=NULL
  schleifendurchlauf = min(length(i_starte), length(i_ende), na.rm = TRUE)
  if(any(which(i_starte[1]<i_ende))){schleifendurchlauf=schleifendurchlauf-length(which(i_starte[1]>i_ende))}else{schleifendurchlauf=schleifendurchlauf}
  for(i in 1:schleifendurchlauf){#){
    starte=i_starte[i]+0
    ende=i_ende[min(which(i_ende>starte))]
    
    if (ende-starte>2){                       # Bildung von NA, wenn Bedingung nicht erfuellt wird
      zehr=DF$O2[starte:ende]
      zehr.zeit = as.vector(minuten[starte:ende])-as.vector(minuten[starte])
      our=rlm( zehr ~ zehr.zeit,    # rlm (): robuste Regression
               psi = psi.bisquare)  # psi.bisquare: Ausreisser werden mit geringerer Bewertungszahl belegt
      coef.our[i]=(our$coefficients[2])*-1    # Umrechnung der negativen Steigung
      if(i==schleifendurchlauf){tour[i]= as.vector(minuten[ende])}else{tour[i]= as.vector(mean(minuten[starte:ende]))}
      # Position, an welcher der Steigungskoeffizient platziert werden soll; letzter Steigungskoeffizient steht am Ende und nicht in der Mitte (für Interpolation einfacher)
      dt.zehr[i] = length(zehr.zeit)          # Laenge der Zehrungsphase
    }
    i=i+1
  }
  OUR   = coef.our
  Zeitpunkt_SVR = DF$Datime[1]+(round(tour)*60)
  DF_SVR = na.omit(data.frame(Zeitpunkt_SVR, OUR))
  colnames(DF_SVR)=c("Datime", "SVR")
  OUR_xts = xts::xts(x = DF_SVR$SVR, order.by = DF_SVR$Datime)
  DynPlot(OUR_xts)
  
  # *.csv wird erzeugt, um Ausreisser (z.B. Probenentnahme) haendisch zu eliminieren 
  # write.table(x = DF_SVR, file = paste("3_Ausgabe/Selektion/Eingang/",Versuchstage[Tag], ".csv", sep = ""),
  #             row.names = FALSE, col.names=FALSE, sep = ";", dec = ",")
  # 
  DF_SVR = read.csv(file = paste("3_Ausgabe/Selektion/Ausgang/", Versuchstage[Tag], ".csv", sep = ""), header = FALSE, sep = ";", dec = ",")
  colnames(DF_SVR) = c("Datime", "SVR")
  Datime = strptime(x = DF_SVR$Datime, format = "%d.%m.%Y %H:%M", tz = "UTC")
  if(any(is.na(Datime))){Datime = strptime(x = DF_SVR$Datime, format = "%Y-%m-%d %H:%M", tz = "UTC")}
  minuten_original = as.vector(difftime(Datime, Datime[1], units = 'mins'))
  SVR_original     = DF_SVR$SVR
  
  #Lineare Interpolation (Minutenwerte)
  XOUT = seq(minuten_original[1], minuten_original[length(minuten_original)], 1)
  SVR_interpoliert = approx(x = minuten_original, y = SVR_original, xout = XOUT, method = "linear")
  
  DF_SVR_interpoliert = data.frame((Datime[1]+(SVR_interpoliert$x*60)), SVR_interpoliert$x, SVR_interpoliert$y)
  colnames(DF_SVR_interpoliert) = c("Datime", "minuten", "SVR")
  
  ##################################################################_
  ##################################################################_
  #----PLOT: Sauerstoffverbrauchsrate ----
  minuten = (DF_SVR_interpoliert$Datime-DF_SVR_interpoliert$Datime[1])/60
  stunden = round(minuten/60, digits = 0)
  if(max(stunden)>25){
    x_aufteilung = stunden*60*10
    x_beschriftung = stunden*10
    plot_height = 7; plot_width = 21
  }else{
    x_aufteilung = stunden*60*2
    x_beschriftung = stunden*2
    plot_height = 7.5; plot_width = 10}
  
  pdf(file = paste("2_Abbildungen/2_Sauerstoffverbrauchsrate/", Versuchstage[Tag] ,".pdf", sep=""), width = plot_width, height = plot_height)
  par(mar=c(5,6,4,2)+0.1, oma = c(0, 0, 0, 2))
  plot(x = DF_SVR_interpoliert$minuten, y = DF_SVR_interpoliert$SVR*60, col="black", type="l",xaxt = "n", ylim = range(0.5, max(DF_SVR_interpoliert$SVR*60)), #xlim = range(0,1415/4),#
       ylab= expression("Verbrauchsrate [mg "* L^-1 *" "* h^-1 *"]" ), xlab="Versuchszeit [h]", #yaxt = "n"
       lwd=2.5, cex.axis=2, cex.lab=2)#, panel.first=abline(h=c(2,3,4,5,10,20), lwd = 1.5,
  grid(nx = NA, ny = NULL, lwd = 2, col="gray50", lty = 2)
  abline(v = x_aufteilung , col="gray50", lty=2, lwd =2)
  lines(x = DF_SVR_interpoliert$minuten, y = DF_SVR_interpoliert$SVR*60, col="black", type="l", lwd=2.5)
  points(x = minuten_original, y = SVR_original*60,  type="p", pch = 21, lwd=2, cex = 3,col="gray80", bg="gray30")
  axis(1,  at=x_aufteilung, labels=x_beschriftung,  cex.axis=2,  las=1, mgp=c(3, 1.3, 0))# Abstand beschriftung zum Text
  legend("topright",cex=2, y=NULL, legend = c("SVR","SVR (linear interpoliert)"),
         inset = 0.05, col = c("gray80", "black"), lty=c(NA, 1), pch=c(21,NA), pt.bg = c("gray30",NA),
         lwd = c(2,2), pt.cex = c(3,NA), bg = alpha("white", 0.7))
  dev.off()
  ##################################################################_
  ##################################################################_
  
  ###############################################################_
  #---Zusammenfassung aller Daten in NA-Matrix
  #Labordaten:
  labor = ReadLABOR(Datei =  "../../1_Daten/2_Optimierter Kohlenstoffabbau - Labor/Laboranalyse/Laboranalyse.xlsx",
                    Registerblatt = Versuchstage[Tag])
  LABOR = FindinLABOR(ReadLABOR_Output = labor, Kriterien = c("1_0"), Parameter=c("CSBmf","CSB","BSB5","NO3.N"), XTS = FALSE,
                      Versuchstag = NA,spectrolyser=FALSE)
  LABOR = data.frame(LABOR[,-2]); colnames(LABOR)[1] = "Datime"
  
  #spectro::lyser-Daten:
  Link_scan = paste("../../1_Daten/2_Optimierter Kohlenstoffabbau - Labor/1_Primaerdaten/", Versuchstage[Tag],"/UVVIS", sep = "")
  Datei_scan = dir(Link_scan)[grep(pattern = ".par", dir(Link_scan))]
  SCAN_par   = ReadSCANtoLIST(Datei = paste(Link_scan, Datei_scan, sep = "/"),
                              format="%Y.%m.%d  %X", dec = ",", XTS = FALSE)
  Datei_scan = dir(Link_scan)[grep(pattern = ".fp", dir(Link_scan))]
  SCAN_fp    = ReadSCANtoLIST(Datei = paste(Link_scan, Datei_scan, sep = "/"),
                              format="%Y.%m.%d  %X", dec = ",", XTS = FALSE)
  if(ncol(SCAN_fp[[2]])>215){SCAN_fp[[2]] = SCAN_fp[[2]][,1:215]} #NaN werden abgeschnitten}
  
  #NA-Matrix
  if((DF_SVR_interpoliert$Datime[1] < SCAN_par[[2]]$Datime[1])){
    Zeit_vec_Anfang = DF_SVR_interpoliert$Datime[1]}else{Zeit_vec_Anfang = SCAN_par[[2]]$Datime[1]}
  
  if((DF_SVR_interpoliert$Datime[length(DF_SVR_interpoliert$Datime)]) > (SCAN_par[[2]]$Datime[length(SCAN_par[[2]]$Datime)])){
    Zeit_vec_Ende = DF_SVR_interpoliert$SVR[length(DF_SVR_interpoliert$SVR)]}else{Zeit_vec_Ende = SCAN_par[[2]]$Datime[length(SCAN_par[[2]]$Datime)]}
  
  Zeit_vec = seq(Zeit_vec_Anfang, Zeit_vec_Ende, 30)
  NA_Matrix = matrix(NA, nrow = length(Zeit_vec), ncol = ncol(DF_SVR_interpoliert)-1 + ncol(LABOR)-1 + ncol(SCAN_par[[2]])-1 + ncol(SCAN_fp[[2]])-2)
  
  Gesamt = data.frame(Zeit_vec, NA_Matrix)
  colnames(Gesamt) = c("Datime", colnames(LABOR)[-1], "SVR", colnames(SCAN_par[[2]])[-1], colnames(SCAN_fp[[2]])[-1])
  #head(Gesamt)
  
  #Suche nach uebereinstimmenden Zeiten in den einzelnen Datensaetzen#
  for (pp in 1:nrow(DF_SVR_interpoliert)) { #Sauerstverbrauchsraten
    POS_O2 = which(Gesamt$Datime == DF_SVR_interpoliert$Datime[pp])
    Gesamt[POS_O2,(ncol(LABOR)+1)] = DF_SVR_interpoliert[pp,-1:-2]
  }
  for (oo in 1:nrow(LABOR)) { #Labordaten
    POS_lab = which(Gesamt$Datime == LABOR$Datime[oo])
    i=2
    for(o in 1:length(LABOR[oo,-1])){
      if(LABOR[oo,-1][o]==0){i=i+1}else{Gesamt[POS_lab,i]=LABOR[oo,-1][o]; i=i+1}
    }
  }
  for (qq in 1:nrow(SCAN_par[[2]])) { #spectro::lyser Daten (*.par)
    POS_scan = which(Gesamt$Datime == SCAN_par[[2]]$Datime[qq])
    Spalten_Pos=seq((ncol(LABOR)+2), (ncol(SCAN_par[[2]])+ncol(LABOR)),1)
    Gesamt[POS_scan, Spalten_Pos] = SCAN_par[[2]][qq,-1]
  }
  for (qqq in 1:nrow(SCAN_fp[[2]])) { #spectro::lyser Daten (*.fp)
    POS_scan = which(Gesamt$Datime == SCAN_fp[[2]]$Datime[qqq])
    Spalten_Pos_fp=seq((ncol(LABOR)+2+ncol(SCAN_par[[2]])-1), (ncol(SCAN_fp[[2]])+ncol(LABOR)+2+ncol(SCAN_par[[2]])-3),1)
    Gesamt[POS_scan, Spalten_Pos_fp] = SCAN_fp[[2]][qqq,-1]
  }
  ####################################################################_
  
  Gesamt$SVR[is.na(Gesamt$SVR)] = 0
  Position_SVR = which(Gesamt$SVR!=0)
  
  SVR_CSB = rep(NA,nrow(Gesamt))
  SVR_AIR = rep(NA,nrow(Gesamt))
  SVR_NO3 = rep(NA,nrow(Gesamt))
  Minutenvektor = rep(NA,nrow(Gesamt))
  Gesamt_3        = cbind(Gesamt[,1], Minutenvektor, Gesamt[,2:6], SVR_CSB, SVR_AIR, SVR_NO3,Gesamt[,-1:-6])
  colnames(Gesamt_3)[1] = "Datime"
  
  if((DF_SVR_interpoliert$Datime[1] < LABOR$Datime[1])){
    Anfang = DF_SVR_interpoliert$Datime[1]}else{Anfang = LABOR$Datime[1]}
  if((Gesamt_3$Datime[Position_SVR[length(Position_SVR)]] > LABOR$Datime[length(LABOR$Datime)])){
    Ende = Gesamt_3$Datime[Position_SVR[length(Position_SVR)]]}else{Ende = LABOR$Datime[length(LABOR$Datime)]}
  
  POSITION_Anfang = which(Gesamt_3$Datime==Anfang)
  POSITION_Ende   = which(Gesamt_3$Datime==Ende)
  
  Differenz=as.numeric(difftime(time1 = Gesamt_3$Datime[POSITION_Ende],time2 = Gesamt_3$Datime[POSITION_Anfang],units = "mins"))
  
  Gesamt_3[POSITION_Anfang:POSITION_Ende,2] = seq(0,Differenz, 0.5)
  
  #SVR sind Mittelwerte fuer einen Zehrabschnitt, die mittig des Abschnitts platziert sind. SVR-Werte muessen zu Beginn und zum Ende des Versuchs aproximiert werden, damit SVR_t0 != NULL ist.
  y = Gesamt_3$SVR[Position_SVR]
  x = Gesamt_3$Minutenvektor[Position_SVR]
  SOLL_y = seq(0, x[length(x)],0.5)
  SVR_approx = approx(x, y, method = "linear", xout = SOLL_y, rule = 2:2)
  
  Gesamt_3[POSITION_Anfang:(POSITION_Ende-1),7] = SVR_approx$y[1:(POSITION_Ende-POSITION_Anfang)]*60
  
  
  ###############################################################_
  #---Sauerstoffhaushalt im Versuchreaktor ----
  #Sauerstoffeintrages ueber die Oberflaeche
  Fakor      = 0.6              #---Faktor: 60% O2-Eintrag in Abwasser im Verhaeltnis zum Klarwasser
  our.air    = 0 # 0.01 # mg L-1 min-1 Annahme: Reaktoren sind verschlossen und werden nur zur kurzen Probenentnahme geoeffnet. Kurze Wiederbelueftung des Inneraums, der innerhalb des Versuchszeitraumes grosser wird, wird vernachlaessigt werden
  Gesamt_3$SVR_AIR[POSITION_Anfang:POSITION_Ende] = our.air * Fakor * 60 # mg L-1 h-1
  
  #Sauerstoffverbrauch durch Nitrifikation:
  if(NO3_N[Tag]==TRUE){# - ermittelt über die Zunahme der Nitrat-Konzentration NO3-N
    DF_NO3.N = na.omit(data.frame(Gesamt_3$Minutenvektor,Gesamt_3$NO3.N))
    DF_NO3.N = DF_NO3.N[which(DF_NO3.N[,2]!=0),]
    Fitall(X = DF_NO3.N[,1], Y = DF_NO3.N[,2]) #---Welche Funktion beschreibt den NO3.N Verlauf am besten?
    
    #Anwendung der besten Fkt.
    POL            = ISI.Toolbox::POLRegression(DF_NO3.N[,1], DF_NO3.N[,2], summary = TRUE)
    Zeitvektor_NO3 = seq(0,DF_NO3.N[nrow(DF_NO3.N),1],0.5)
    NO3.N_x_inter  = data.frame(x = Zeitvektor_NO3)
    NO3.N_y_inter  = predict(POL, newdata = NO3.N_x_inter, se.fit = FALSE, interval = "confidence")
    
    ##################################################################_
    ##################################################################_
    #----PLOT: NH3-Konzentration ----
    minuten = (Gesamt$Datime-Gesamt$Datime[1])/60
    stunden = round(minuten/60, digits = 0)
    if(max(stunden)>25){
      x_aufteilung = stunden*60*10
      x_beschriftung = stunden*10
      plot_height = 7; plot_width = 21
    }else{
      x_aufteilung = stunden*60
      x_beschriftung = stunden
      plot_height = 7.5; plot_width = 10}
    pdf(file = paste("2_Abbildungen/3_NO3Konzentration/", Versuchstage[Tag],".pdf", sep=""), width = plot_width, height = plot_height)
    par(mar=c(5,6,4,2)+0.1, oma = c(0, 0, 0, 2))
    plot(Zeitvektor_NO3, c(NO3.N_y_inter[,1]), type = "l", ylab = expression("Konzentration [mg "* L^-1*"]" ),lwd=2, xlab="Versuchszeit [h]",
         cex.axis=2, xaxt = "n", cex.lab=2)
    grid(nx = NA, ny = NULL, lwd = 2, col="gray50", lty = 2)
    stunden = unique(round(Gesamt_3$Minutenvektor/60, digits = 0))*10
    abline(v = stunden*60 , col="gray50", lty=2, lwd =2)
    polygon(c(Zeitvektor_NO3,rev(Zeitvektor_NO3)),c(c(NO3.N_y_inter[,2]),rev(c(NO3.N_y_inter[,3]))),
            col = "grey75", border = FALSE)
    lines(Zeitvektor_NO3, c(NO3.N_y_inter[,1]), col="black", lwd=2.5)
    lines(Zeitvektor_NO3, c(NO3.N_y_inter[,2]), col = "grey65", lwd=1)
    lines(Zeitvektor_NO3, c(NO3.N_y_inter[,3]), col= "grey65", lwd=1)
    points(DF_NO3.N[,1], DF_NO3.N[,2],  type="p", pch = 21, lwd=2, cex = 3,col="gray80", bg="gray30")
    box()
    axis(1, at=x_aufteilung,labels=x_beschriftung,cex.axis=2, las=1, mgp=c(3, 1.3, 0))
    legend("topleft",cex=2, y=NULL, legend = c(expression(NH[3] *"-N (Stichprobe)"),expression(NH[3] *"-N (quadratisch interpoliert)"), "Konfidenzintervall"),
           inset = 0.05, col = c("gray80", "black", "grey65"), lty=c(NA, 1), pch=c(21,NA,22), pt.bg = c("gray30",NA, "grey75"),
           lwd = c(2,2,2), pt.cex = c(3,NA,5), bg = alpha("white", 0.6))
    dev.off()
    ##################################################################_
    ##################################################################_
    
    #interpolierte Werte werden in die "GESAMT"-Variable geschrieben
    Gesamt_3$NO3.N[POSITION_Anfang:POSITION_Ende] = NO3.N_y_inter[,1]
    #plot(Gesamt_3$NO3.N)
    
    Gesamt_3$SVR_NO3[-1] = diff((Gesamt_3$NO3.N)*4.33)*60       #---O2, der infolge der Nitrifikation pro dt (=1min) verbraucht wird
    Gesamt_3$SVR_NO3[is.na(Gesamt_3$SVR_NO3)] = 0
    
    Gesamt_3$SVR_CSB = (Gesamt_3$SVR ) - Gesamt_3$SVR_NO3 + Gesamt_3$SVR_AIR# O2-Verbrauch infolge der Nitrifikation wird vom Gesamt-OUR abgezogen
  }else{
    Gesamt_3$SVR_CSB = Gesamt_3$SVR + Gesamt_3$SVR_AIR      # kein O2-Verbrauch durch Nitrifikation
    Gesamt_3$SVR_NO3[POSITION_Anfang:POSITION_Ende] = 0
  }
  
  ###############################################################_
  # head(Gesamt_3) ----
  # Tag=7
  Dateiname_Ergebnis = paste("3_Ausgabe/Datenzusammenfassung/3/", Versuchstage[Tag], "_Respirationsversuch.txt", sep = "")
  #write.table(x = Gesamt_3, file = Dateiname_Ergebnis, sep = "\t", col.names = TRUE, row.names = FALSE, dec = ".")
  
  Gesamt_3  = read.table(file = Dateiname_Ergebnis, sep="\t", header = TRUE, dec = ".")
  Datime   = strptime(Gesamt_3$Datime, format="%Y-%m-%d %H:%M:%S", tz="UTC")
  Gesamt_3 = data.frame(Datime, Gesamt_3[,-1])
  
  Gesamt_3_SVR = subset(Gesamt_3, Gesamt_3$SVR!=0)
  Gesamt_3_SVR = Gesamt_3_SVR[,1:10]
  
  Gesamt_4_list = list(Gesamt_3_SVR, Gesamt_3)
  
  DF_SVR = read.csv(file = paste("3_Ausgabe/Selektion/Ausgang/", Versuchstage[Tag], ".csv", sep = ""), header = FALSE, sep = ";", dec = ",")
  colnames(DF_SVR) = c("Datime", "SVR")
  Datime = strptime(x = DF_SVR$Datime, format = "%d.%m.%Y %H:%M", tz = "UTC")
  if(any(is.na(Datime))){Datime = strptime(x = DF_SVR$Datime, format = "%Y-%m-%d %H:%M", tz = "UTC")}
  minuten_original = as.vector(difftime(Datime, Datime[1], units = 'mins'))
  SVR_original     = DF_SVR$SVR
  
  #Uebergaenge innerhalb der Versuche (manuell herausgesucht; fuer gradientenbasierte Selektion Versuche zu heterogen!)
  #,"2013-03-18 12:14:00"
  Ss_Ende = strptime(c("2013-03-11 17:18:00","2013-04-02 19:58:00","2013-04-09 16:18:00","2014-04-08 16:08:00",
                       "2014-04-28 20:51:00","2014-05-05 20:02:00","2018-02-13 17:19:00","2018-03-05 17:05:00","2019-02-21 14:01:00",
                       "2019-02-25 13:24:00","2019-02-26 13:11:00", "2019-03-05 11:35:00", "2019-04-08 12:52:00","2019-04-10 15:03:00",
                       "2020-03-05 14:39:00"), format = "%Y-%m-%d %H:%M:%S", tz = "UTC")
  #,"2013-03-20 18:27:00"
  Xs_Ende = strptime(c("2013-03-13 16:12:00","2013-04-05 06:26:00","2013-04-11 17:14:00","2014-04-11 15:34:00",
                       "2014-04-30 15:20:00","2014-05-09 02:25:00","2018-02-17 23:56:00","2018-03-07 10:13:00","2019-02-21 14:17:00",
                       "2019-02-25 14:20:00","2019-02-26 13:59:00","2019-03-05 14:09:00","2019-04-09 07:58:00","2019-04-11 08:58:00",
                       "2020-03-06 10:34:00"), format = "%Y-%m-%d %H:%M:%S", tz = "UTC")
  Pos_Ss_Ende = which(Gesamt_4_list[[1]]$Datime == Ss_Ende[Tag])
  Pos_Xs_Ende = which(Gesamt_4_list[[1]]$Datime == Xs_Ende[Tag])
  if(is.integer0(Pos_Xs_Ende)){Pos_Xs_Ende = nrow(Gesamt_4_list[[1]])}else{}
  Pos_Versuchsende = nrow(Gesamt_4_list[[1]])
  
  #Ermittlung der Regressionsfunktion (Wentzel):
  Position_original=NULL
  for (iii in minuten_original) {
    pos = which(Gesamt_4_list[[1]]$Minutenvektor==iii)
    Position_original=c(Position_original, pos)
  }
  Max_Pos = which(Gesamt_4_list[[1]]$SVR[Position_original] == max(Gesamt_4_list[[1]]$SVR[Position_original], na.rm = TRUE)) # es wird nach dem Max. der SVR_new-raten gesucht, um Exp.fkt zu bestimmen
  ##########
  #Hier kann ein gezielter Zeitwert eingegeben werden, wenn das Maximum nicht zum korekten Ergebnis fuehrt
  #zeitpunkt mit Dynplot suchen, eingeben und nach dem "Max_Pos" suchen
  #DynPlot(X = xts(x = Gesamt_4_list[[1]]$SVR[Position_original], order.by = Gesamt_4_list[[1]]$Datime[Position_original]))
  #Max_Pos = which(Gesamt_4_list[[1]]$Datime == "2013-03-11 15:57:00") # manuelle Auswahl der SVR, um Exp.fkt zu bestimmen
  ##########
  y = Gesamt_4_list[[1]]$SVR[Position_original][1:Max_Pos]
  x = Gesamt_4_list[[1]]$Minutenvektor[Position_original][1:Max_Pos]/60
  reg     = lm(log(y) ~ x)
  sum_reg = summary(reg)
  ##################################################################_
  ##################################################################_
  #PLOTTEN DER SVR_new-AUFTEILUNG
  pdf(file = paste("2_Abbildungen/4_Fraktionenaufteilung/Kurvenfunktion/", Versuchstage[Tag],".pdf", sep=""),
      width = 10, height = 7.5)
  par(mar=c(5,6,4,2)+0.1, oma = c(0, 0, 0, 2))
  
  #Plot der Regressionsfunktion
  plot(x, reg$fitted.values, ylab = expression("ln SVR [mg "* L^-1* h^-1*"]" ), xlab = "Versuchszeit [h]", type = "n", pch = 21, col = alpha("tomato3", 1), cex=2,
       cex.axis=2, cex.lab=2, lwd=2, bg = alpha("tomato3", 0.8))
  grid(nx = NULL, ny = NULL, lwd = 2, col="gray50", lty = 2)
  #abline(v = x_aufteilung , col="gray50", lty=2, lwd =2)
  
  points(x,log(y), col=alpha("black", 1), pch = 23, cex=2,lwd =2, bg = alpha("black", 0.7))
  points(x, reg$fitted.values, pch = 21, col = alpha("tomato3", 1), cex=2, cex.lab=2, lwd=2, bg = alpha("tomato3", 0.8))
  
  xxx=max(x)*0.66
  yyy=min(reg$fitted.values)*1.2
  legend("topleft",cex=2, y=NULL, title="Sauerstoffverbrauchsrate (SVR)", adj = 0, legend = c("Berechnung", "Messung"),
         inset = 0.05, col = c(alpha("tomato3", 1), alpha("black", 1)), lty=NA, pch=c(21,23), pt.bg = c(alpha("tomato3", 0.8), alpha("black", 0.7)),
         lwd = c(2,2), pt.cex = 2, bg = alpha("white", 0.6))
  
  legend("bottomright",cex=1.5, y=NULL, legend=paste("lnSVR = ", round(reg$coefficients[1], digits = 3)," + ",round(reg$coefficients[2], digits = 3)," * t \n", " R2 = ", round(sum_reg$r.squared, digits = 3)), adj = 0,
         inset = 0.05, bg = alpha("white", 0.9), box.lwd = 0,box.col = "gray80")
  
  dev.off()
  ##################################################################_
  ##################################################################_
  
  #Regressionsergebnis:
  Intercept = reg$coefficients[1]
  Steigung  = reg$coefficients[2]
  
  #Bestimmen der dem Xs zuzuordnende SVR -> SVR_XS
  Y_ZH = 0.666
  b_H  = 0.62
  Z_BH_0 = (exp(Intercept)*24)/(((1-Y_ZH)/Y_ZH) * ((Steigung*24)+b_H)) #Anfangskonzentration der heterotrophen Biomasse
  K_mp = as.numeric(((Gesamt_4_list[[1]]$SVR)[Pos_Ss_Ende]*24)/(((1-Y_ZH)/Y_ZH)*Z_BH_0*exp(Steigung * Gesamt_4_list[[1]]$Minutenvektor[Pos_Ss_Ende]/60)))
  mu_H = (Steigung * 24) - K_mp + b_H
  SVR_Xs = (((1-Y_ZH)/Y_ZH) * K_mp * Z_BH_0 * exp((mu_H + K_mp - b_H) * (Gesamt_4_list[[1]]$Minutenvektor[1:Pos_Ss_Ende]/60/24)))/24
  
  #Zusammenfassung der SVR-Daten: vereinfachter Ansatz und Ansatz von Wentzel
  Tab       = subset(data.frame(Gesamt_4_list[[1]]$Datime, Gesamt_4_list[[1]]$Minutenvektor, Gesamt_4_list[[1]]$SVR_CSB), Gesamt_4_list[[1]]$Minutenvektor!="NA")
  Tab = cbind(Tab, 0, 0, 0, 0, 0, 0)
  colnames(Tab) = c("Zeit", "Minuten", "SVR_gesamt", "SVR_SS", "SVR_XS", "SVR_endogen")
  
  #Wentzel-Ansatz: Der Sauerstoffverbrauch wird im ersten Abschnitt verursacht durch SS und XS
  Tab$SVR_SS[1:Pos_Ss_Ende] = Gesamt_4_list[[1]]$SVR[1:Pos_Ss_Ende]-SVR_Xs
  Tab$SVR_XS[1:Pos_Xs_Ende] = c(SVR_Xs, Tab$SVR_gesamt[(Pos_Ss_Ende+1):Pos_Xs_Ende])
  if(Pos_Xs_Ende==Pos_Versuchsende){}else{Tab$SVR_endogen[(Pos_Xs_Ende+1):Pos_Versuchsende] = Tab$SVR_gesamt[(Pos_Xs_Ende+1):Pos_Versuchsende]}
  
  dim(Tab); dim(Gesamt_4_list[[1]])
  
  #-#-#-# Integration des OUR-Graphen ----
  #jeder Abschnitt wird mit einer Schleife einzelnd betrachtet. Verbindung zwischen den einzelnen Punkten sind linear.
  #als Resultat der Schleife erhaelt der Anwender einen Vector mit den Abschnittsintegralen.
  
  dcsb_SS=0; dcsb_XS=0; dcsb_endogen=0
  dcsb_gesamt=0; dges=0; dair=0; dno3=0
  for(i in 2:nrow(Tab)){
    dcsb_SS[i]=1/2 * (Tab$SVR_SS[i-1]+Tab$SVR_SS[i]) * (Tab$Minuten[i]/60 - Tab$Minuten[i-1]/60) #Flaecheninhalt eines Trapezes F_i = 1/2 * (f(x_i)+f(x_i-1)) * h
    dcsb_XS[i]=1/2 * (Tab$SVR_XS[i-1]+Tab$SVR_XS[i]) * (Tab$Minuten[i]/60 - Tab$Minuten[i-1]/60) #Flaecheninhalt eines Trapezes F_i = 1/2 * (f(x_i)+f(x_i-1)) * h
    dcsb_endogen[i]=1/2 * (Tab$SVR_endogen[i-1]+Tab$SVR_endogen[i]) * (Tab$Minuten[i]/60 - Tab$Minuten[i-1]/60) #Flaecheninhalt eines Trapezes F_i = 1/2 * (f(x_i)+f(x_i-1)) * h
    
    dcsb_gesamt[i]=1/2 * (Tab$SVR_gesamt[i-1]+Tab$SVR_gesamt[i]) * (Tab$Minuten[i]/60 - Tab$Minuten[i-1]/60) #Flaecheninhalt eines Trapezes F_i = 1/2 * (f(x_i)+f(x_i-1)) * h
    
    dno3[i]=((Gesamt_4_list[[1]]$SVR_NO3[i-1] + Gesamt_4_list[[1]]$SVR_NO3[i])/2) * (Gesamt_4_list[[1]]$Minutenvektor[i]/60 - Gesamt_4_list[[1]]$Minutenvektor[i-1]/60)
    dair[i]=((Gesamt_4_list[[1]]$SVR_AIR[i-1] + Gesamt_4_list[[1]]$SVR_AIR[i])/2) * (Gesamt_4_list[[1]]$Minutenvektor[i]/60 - Gesamt_4_list[[1]]$Minutenvektor[i-1]/60)
    dges[i]=((Gesamt_4_list[[1]]$SVR[i-1] + Gesamt_4_list[[1]]$SVR[i])/2) * (Gesamt_4_list[[1]]$Minutenvektor[i]/60 - Gesamt_4_list[[1]]$Minutenvektor[i-1]/60)
    
    i=i+1
  }#; nrow(Gesamt_5_list[[1]])
  dcsb_SS[which(dcsb_SS<0)]=0; dcsb_XS[which(dcsb_XS<0)]=0; dcsb_endogen[which(dcsb_endogen<0)]=0
  dcsb_gesamt[which(dcsb_gesamt<0)]=0;dno3[which(dno3<0)]=0;dair[which(dair<0)]=0;dges[which(dges<0)]=0
  
  dcsb_SS=NA_in_0(dcsb_SS); dcsb_XS=NA_in_0(dcsb_XS); dcsb_endogen=NA_in_0(dcsb_endogen)
  dcsb_gesamt=NA_in_0(dcsb_gesamt);dno3=NA_in_0(dno3);dair=NA_in_0(dair);dges=NA_in_0(dges)
  
  tab = data.frame(cbind(dcsb_gesamt,dcsb_SS,dcsb_XS,dcsb_endogen,dno3,dair,dges))
  
  print(paste("O2-Verbrauch gesamt",round(sum(na.omit(dges)), digits = 2)," mg L-1"))
  print(paste("O2-Verbrauch durch C-Abbau",round(sum(na.omit(dcsb_gesamt)), digits = 2)," mg L-1"))
  print(paste("O2-Verbrauch durch NH3-Oxidation",round(sum(na.omit(dno3)), digits = 2)," mg L-1"))
  print(paste("O2-Eintrag durch Wiederbelueftung",round(sum(na.omit(dair)), digits = 2)," mg L-1"))
  
  ###############################################################_
  #---PRUEFUNG DER RESPIRATION: 
  #Wie viel abgebauter CSB kann mit der SVR beschrieben werden?
  #CSB-Recovery (Wentzel et al. (1995)) = eine Art Gueteparameter des Versuchs (sollte um 1 liegen)
  
  DF_rec = subset(cbind(Gesamt_4_list[[1]]$Minutenvektor,Gesamt_4_list[[1]]$CSB, Gesamt_4_list[[1]]$SVR),
                  (!is.na(Gesamt_4_list[[1]]$CSB))&(Gesamt_4_list[[1]]$SVR!=0))
  
  Ende_labSVR_pos_1    = which(Gesamt_4_list[[1]]$Datime == Gesamt_4_list[[1]][DF_rec[nrow(DF_rec),1],1])
  Ende_labSVR_pos_2ges = which(Gesamt_4_list[[2]]$Datime == Gesamt_4_list[[1]][DF_rec[nrow(DF_rec),1],1])
  
  ##################################################################_
  #----PLOT: SVR-Aufteilung (nach Wentzel) ----
  pdf(file = paste("2_Abbildungen/4_Fraktionenaufteilung/", Versuchstage[Tag],".pdf", sep=""), width = plot_width, height = plot_height)
  par(mar=c(5,6,4,2)+0.1, oma = c(0, 0, 0, 2))
  xx = Gesamt_4_list[[1]]$Minutenvektor; yy = Gesamt_4_list[[1]]$SVR
  plot(xx, yy, type = "n", ylab = expression("SVR [mg "* L^-1* h^-1*"]" ),
       xlab="Versuchszeit [h]", cex.axis=2, xaxt = "n", cex.lab=2)
  grid(nx = NA, ny = NULL, lwd = 2, col="gray50", lty = 2)
  stunden = unique(round(xx, digits = 0))*2
  abline(v = x_aufteilung , col="gray50", lty=2, lwd =2)
  
  if(is.na(Steigung)){
    polygon(c(xx[1:Pos_Ss_Ende],c(xx[Pos_Ss_Ende],xx[1])), c(yy[1:Pos_Ss_Ende],c(yy[Pos_Ss_Ende],0)),
            col = alpha("tomato3", 0.8), border = alpha("tomato3", 1),lwd = 2)
    polygon(c(xx[1], xx[Pos_Ss_Ende],xx[Pos_Ss_Ende:Pos_Xs_Ende],rev(xx[Pos_Ss_Ende:Pos_Xs_Ende]),xx[Pos_Ss_Ende],xx[1]), c(0,yy[Pos_Ss_Ende],yy[Pos_Ss_Ende:Pos_Xs_Ende],rep(0,(Pos_Xs_Ende-Pos_Ss_Ende+1)),rep(0,Pos_Ss_Ende)),
            col = alpha("steelblue4", 0.8), border = alpha("steelblue4", 1),lwd = 2 )
  }else{
    polygon(c(xx[1:Pos_Ss_Ende],rev(xx[1:Pos_Ss_Ende])),c(yy[1:Pos_Ss_Ende],rev(SVR_Xs)),
            col = alpha("tomato3", 0.8), border = alpha("tomato3", 1),lwd = 2)
    polygon(c(xx[1:Pos_Ss_Ende],xx[Pos_Ss_Ende:Pos_Xs_Ende],rev(xx[Pos_Ss_Ende:Pos_Xs_Ende]),rev(xx[1:Pos_Ss_Ende])),c(SVR_Xs,yy[Pos_Ss_Ende:Pos_Xs_Ende],rep(0,(Pos_Xs_Ende-Pos_Ss_Ende+1)),rep(0,Pos_Ss_Ende)),
            col = alpha("steelblue4", 0.8), border = alpha("steelblue4", 1),lwd = 2 )
  }
  axis(1, at=x_aufteilung, labels=x_beschriftung,  cex.axis=2, las=1, mgp=c(3, 1.3, 0))
  
  if(Pos_Xs_Ende==Pos_Versuchsende){
    legend("topright",cex=2, y=NULL, title="Respiration infolge des Abbaus von...", adj = 0, legend = c("gelösten und", "partikulären CSB."),
           inset = 0.05, col = c(alpha("tomato3", 1), alpha("steelblue4", 1)), lty=NA, pch=22, pt.bg = c(alpha("tomato3", 0.8), alpha("steelblue4", 0.8)),
           lwd = c(2,2), pt.cex = 5, bg = alpha("white", 0.6))
  }else{
    polygon(c(xx[(Pos_Xs_Ende+1):Pos_Versuchsende],rev(xx[(Pos_Xs_Ende+1):Pos_Versuchsende])),c(yy[(Pos_Xs_Ende+1):Pos_Versuchsende],rep(0,length(yy[(Pos_Xs_Ende+1):Pos_Versuchsende]))),
            col = alpha("steelblue4", 0.4), border = alpha("steelblue4", 0.8),lwd = 2)
    legend("topright",cex=2, y=NULL, title="Respiration infolge des Abbaus von...", adj = 0, legend = c("gelösten und", "partikulären CSB.", "Endogene Respiration"),
           inset = 0.05, col = c(alpha("tomato3", 1), alpha("steelblue4", 1), alpha("steelblue4", 0.8)), lty=NA, pch=22, pt.bg = c(alpha("tomato3", 0.8), alpha("steelblue4", 0.8), alpha("steelblue4", 0.4)),
           lwd = c(2,2,2), pt.cex = 5, bg = alpha("white", 0.6))
  }
  box()
  dev.off()
  ##################################################################_
  ##################################################################_
  
  Gesamt_4.1_list_name = c(colnames(Gesamt_4_list[[1]])[7:10], colnames(tab))
  Gesamt_4.1_list     = cbind(Gesamt_4_list[[1]][7:10], tab)
  
  Pos_in_Liste = which(Gesamt_4_list[[2]]$Datime == Gesamt_4_list[[1]]$Datime[1])
  
  MAT = matrix(data = NA, nrow = nrow(Gesamt_4_list[[2]]), ncol = ncol(Gesamt_4.1_list))
  
  Gesamt_4 = cbind(Gesamt_4_list[[2]][,1:10], MAT, Gesamt_4_list[[2]][,-1:-10])
  
  col.nr = seq(1,ncol(Gesamt_4.1_list),1) + 10
  row.nr = seq(1,nrow(Gesamt_4_list[[1]]),1) + Pos_in_Liste - 1
  Gesamt_4[row.nr, col.nr] = Gesamt_4.1_list
  col.name = c(colnames(Gesamt_4_list[[2]][,1:10]),Gesamt_4.1_list_name,colnames(Gesamt_4_list[[2]][,-1:-10]))
  colnames(Gesamt_4) = col.name
  
  Dateiname_Ergebnis = paste("3_Ausgabe/Datenzusammenfassung/4/", Versuchstage[Tag], "_Respirationsversuch.txt", sep = "")
  write.table(x = Gesamt_4, file = Dateiname_Ergebnis, sep = "\t", col.names = TRUE, row.names = FALSE, dec = ".")
  
  Gesamt_4  = read.table(file = Dateiname_Ergebnis, sep="\t", header = TRUE, dec = ".")
  Datime   = strptime(Gesamt_4$Datime, format="%Y-%m-%d %H:%M:%S", tz="UTC")
  Gesamt_4 = data.frame(Datime, Gesamt_4[,-1])
  
  Gesamt_4_SVR = subset(Gesamt_4, Gesamt_4$SVR!=0)
  Gesamt_4_SVR = Gesamt_4_SVR[,1:24]
  Gesamt_5_list = list(Gesamt_4_SVR, Gesamt_4)
  
  Ss_Ende = strptime(c("2013-03-11 17:18:00","2013-04-02 19:58:00","2013-04-09 16:18:00","2014-04-08 16:08:00",
                       "2014-04-28 20:51:00","2014-05-05 20:02:00","2018-02-13 17:19:00","2018-03-05 17:05:00","2019-02-21 14:01:00",
                       "2019-02-25 13:24:00","2019-02-26 13:11:00", "2019-03-05 11:35:00", "2019-04-08 12:52:00","2019-04-10 15:03:00",
                       "2020-03-05 14:39:00"), format = "%Y-%m-%d %H:%M:%S", tz = "UTC")
  
  Xs_Ende = strptime(c("2013-03-13 16:12:00","2013-04-05 06:26:00","2013-04-11 17:14:00","2014-04-11 15:34:00",
                       "2014-04-30 15:20:00","2014-05-09 02:25:00","2018-02-17 23:56:00","2018-03-07 10:13:00","2019-02-21 14:17:00",
                       "2019-02-25 14:20:00","2019-02-26 13:59:00","2019-03-05 14:09:00","2019-04-09 07:58:00","2019-04-11 08:58:00",
                       "2020-03-06 10:34:00"), format = "%Y-%m-%d %H:%M:%S", tz = "UTC")
  Pos_Ss_Ende = which(Gesamt_5_list[[1]]$Datime == Ss_Ende[Tag])
  Pos_Xs_Ende = which(Gesamt_5_list[[1]]$Datime == Xs_Ende[Tag])
  if(is.integer0(Pos_Xs_Ende)){Pos_Xs_Ende = nrow(Gesamt_5_list[[1]])}else{}
  Pos_Versuchsende = nrow(Gesamt_5_list[[1]])
  
  ###############################################################_
  #---Labor-Daten ----
  #Berechnung BCOD (Roeleveld et al., 2002)----
  BCOD = rep(x = NA, nrow(Gesamt_5_list[[1]]))
  if(any(is.na(Gesamt_5_list[[1]]$BSB5)==FALSE)){
    day   = 5    #[d]
    f_BOD = 0.15 #[-] (0.1-0.2) Korrekturfaktor; analog zum inerten Anteil aus Biomassezerfall (fip)
    k_BOD = 0.21 #[d-1] Geschwindigkeitskonstante
    BOD_t = na.omit(Gesamt_5_list[[1]]$BSB5)#[mg L-1]
    BOD_t_pos = which(!is.na(Gesamt_5_list[[1]]$BSB5))
    
    BOD_tot = 1/(1-exp(-k_BOD * day)) * BOD_t
    bcod = 1/(1-f_BOD) * BOD_tot
    BCOD[BOD_t_pos] = bcod
  }else{
    print("Kein BSB_5 Wert vorhanden!")
  }
  Gesamt_5_list[[1]] = data.frame(cbind(Gesamt_5_list[[1]][,1:5], BCOD, Gesamt_5_list[[1]][,6:ncol(Gesamt_5_list[[1]])])) #Tabelle wird um den Parameter BCOD erweitert
  
  
  ###############################################################_
  #---Modellparameter ----
  Parameter_kal = c("O2_mean", "Y_h", "f_xi")
  
  #Kalibrierparameter
  Parameter_Y_h     = seq(from = 0.38, to = 0.8, by = 0.0575)#aus Jeppsson (1996)
  Parameter_f_xi    = seq(from = 0.08, to = 0.2, by = 0.03) #Kappeler und Gujer 1992 sowie Henze et al. 2000
  
  #---Activated Sludge Modell ----
  m=1; n=1; i=1; o=1; D_F=NULL; CSB_ges_alle=NULL
  Zusammenfassung_pruefparameter = NULL
  Zusammenfassung_kalibrierkombination = NULL
  
  #Xs-Beschreibung: Xs beschreibt die theoretische Menge an partikulaeren Substrat, das mit der detektitierten SVR abgebaut werden koennte.
  #                 wenn kein O2 vorhanden, kann kein Xs abgebaut werden
  #                 Durch den Biomassezerfall entsteht in jedem Zeitschritt zusaetzlich abbaubares Substrat (Xp)
  #                 die Folge ist eine kleinere Differenz zwischen Anfangs- und Endkonstration (XsRW) wie die Xs-Kurve vermuten laesst
  #                 Xp kummulativ dargestellt ergibt die Differnez zwischen Xs und XsRW
  #Schleifen mit Parametervariationen
  
  #SCHLEIFEN DER KALIBRIERPARAMETER --> NSGA-II ???
  for (o in 1:length(Parameter_f_xi)) {
    for (n in 1:length(Parameter_Y_h)) {
      #################################################################_
      #----Berechnung CSB-Fraktionen mit Belebtschlamm Modell (Activated Sludge No.1)----
      #Heterotropher Yield Koeffizient (Wachstumsrate)
      yH    = Parameter_Y_h[n]  # [g CSB / g CSB] Zellertragsrate aktiver heterotropher Biomasse
      bh    = 0.62/(60*24)              # [-]      maximale Zerfallsrate
      
      ####Welche Zerfallsprodukte entstehen?
      #ASM1 betrachtet im Gegensatz zum ASM3 den Zerfall
      fXi = Parameter_f_xi[o]  # [g CSB / g CSB] Anteil der Biomasse, der zu inert-partikulaeren Stoffen fuehrt
      fXp = 1 - fXi # [g CSB / g CSB] Anteil der Biomasse, der zu abbaubaren-partikulaeren Stoffen fuehrt (Henze 1992)
      
      #Nullvektoren mit Belegung des ersten Wertes
      Xh=NULL; dXh_b=NULL; dSsCSB=NULL; dXs=NULL; dXh_g=NULL; dXp=NULL; dXip=NULL; dXsRW=NULL; Si=NULL
      Xh[1]=Z_BH_0 # Biomasse nach Wentzel et al. 1995 vorhanden
      dXh_b[1]=0; dSsCSB[1]=0; dXs[1]=0; dXh_g[1]=0; dXp[1]=0; dXip[1]=0; dXsRW[1]=0;
      Si[1] = 0.9*as.numeric(na.omit(Gesamt_5_list[[1]]$CSBmf))[length(na.omit(Gesamt_5_list[[1]]$CSBmf))] #Faktor 0.9 (Roeleveld et al. 2002)
      #########################################
      
      k=2
      for (k in 2:nrow(Gesamt_5_list[[1]])){#length(dcsb_gesamt)){
        if (k <= Pos_Ss_Ende){
          #Erstes Atmungsniveau (hoch)
          #Annahme: es wird anteilig geloestes Substrat (Ss) und hydrolisiertes Substrat (Xs) verbraucht;
          #         Hydrolyse (1 Xs -> 1 Ss) und Verbrauch geschehen im selben Zeitschritt t
          #         Biomasse (Xh) wird aufgebaut und zerfaellt (t-1) in Xp und Xip
          #         Xp entsteht und wird im selben Zeitschritt t verbraucht
          #         Xs wird verbraucht sobald Xp innerhalb des Zeitschrittes t vollstaendig verbraucht ist
          #         XsRW faellt -> Umsatz zum Zeitpkt t ist negativ
          
          dSsCSB[k] = Gesamt_5_list[[1]]$dcsb_SS[k]/(1-yH) #SVR wird anteilig Ss zugeordnet
          dXs[k]    = Gesamt_5_list[[1]]$dcsb_XS[k]/(1-yH)
          #Biomasse Xh
          dXh_g[k] = dSsCSB[k] * yH                #Biomassewachstum durch Substrat: dt infolge des gelösten C-Abbaus
          dXh_b[k] = Xh[k-1] * bh                  #Biomassezerfall/ Endogene Atmung
          Xh[k]      = Xh[k-1] - dXh_b[k] + dXh_g[k] #Resultierende Biomasse: Ergebnis aus vorhandener, gewachsener und infolge der Endogenen Atmung zerfallener Biomasse
          #Zerfallsprodukte von Xh (Biomasse)
          dXp[k]     = dXh_b[k-1] * fXp                #Xs aus Biomassezerfall
          dXip[k]    = dXh_b[k]   * fXi                #Xi aus Biomassezerfal
          dXsRW[k]   = dXs[k]     - dXp[k]   #dXsRW[k]   = dXs[k]
          if( dXp[k]<dXs[k] ){ dXp[k]=0 }else{ dXp[k]=dXp[k]-dXs[k] } #wenn Xp nicht vollstaendig verbraucht wird ist Xp nicht 0
          Si[k]=Si[1]+((as.numeric(na.omit(Gesamt_5_list[[1]]$CSBmf))[length(na.omit(Gesamt_5_list[[1]]$CSBmf))]-Si[1])/nrow(Gesamt_5_list[[1]])*k)
          
        }else if ( (k<Pos_Xs_Ende) & (k>Pos_Ss_Ende) ){    #Zweites Atmungsniveau (mittel)
          #
          #Annahme: es wird hydrolisiertes Substrat (Xs) verbraucht;
          #         Hydrolyse (1 Xs -> 1 Ss) und Verbrauch geschehen im selben Zeitschritt t
          #         Biomasse (Xh) wird aufgebaut und zerfaellt (t-1) in Xp und Xip
          #         Xp entsteht und wird im selben Zeitschritt t verbraucht
          #         Xs wird verbraucht sobald Xp innerhalb des Zeitschrittes t vollstaendig verbraucht ist
          #         XsRW faellt -> Umsatz zum Zeitpkt t ist negativ
          
          dSsCSB[k]  = 0                    #Ss ist AUFGEBRAUCHT und im zweiten Atmungsniveau nicht mehr vorhanden
          dXs[k]     = Gesamt_5_list[[1]]$dcsb_XS[k]/(1-yH)
          
          dXh_g[k]   = dXs[k]  * yH
          dXh_b[k]   = Xh[k-1] * bh
          Xh[k]      = Xh[k-1] - dXh_b[k] + dXh_g[k]
          
          dXp[k]     = dXh_b[k] * fXp
          dXip[k]    = dXh_b[k] * fXi       #Xi zum Zeitpunkt t_0 ist NULL. Xi geht ausschliesslich aus Zerfallsprodukten X_BH hervor
          #fuer die Berechung des XS im Rohwasser wird unterstellt, zuerst die Zerfallsprodukte gefressen werden
          dXsRW[k]   = dXs[k] - dXp[k]      #Menge an Xs, die unter Beruecksichtigung der Neubildung verbraucht wird
          if(dXp[k]<dXs[k]){ dXp[k]=0 }else{ dXp[k]=dXp[k]-dXs[k] } #wenn Xp nicht vollstaendig verbraucht wird ist Xp nicht 0
          Si[k]=Si[1]+((as.numeric(na.omit(Gesamt_5_list[[1]]$CSBmf))[length(na.omit(Gesamt_5_list[[1]]$CSBmf))]-Si[1])/nrow(Gesamt_5_list[[1]])*k)
        }else{                              #Drittes Atmungsniveau (schwach -> ausschliesslich auf Endogene Atmung zurueckzufuehren)
          #
          #Annahme: Gesamter Ss und Xs aus dem Abwasser zum Zeitpunkt t_0 sind veratmet -> Umsatz zum Zeitpkt. t betraegt 0
          #         Substrat fuer Biomassewachstum wird auschliesslich aus dem Xp-Umsatz zum Zeitp. t bezogen.
          #         Xp-Umsatz zum Zeitpkt. t setzt sich zusammen aus dem Zerfallsprodukt der Biomasse abzueglich
          #         der Menge, die fuer den Biomasseaufbau verwendet wird.
          #         Verwendetes Substrat ...- dXh_g[k]*1/yH -> Subtraktion durch yH (Rückrechnung), um erforderliche Substratmenge zu ermitteln.
          #         Zerfall zum Zeitpkt. t wird berechnet aus Konz. het. Biomasse zum Zeitpkt. t-1
          #         effektiver Anteil an Xip aus den Zerfallsprodukten der het. Biomasse steigt durch den Verbrauch
          #         von Xp -> fi/[(1-yH)*(1-fi)] -> 0.08/[(1 - 0.67)*(1 - 0.08)] = 0.263
          #wenn kein O2 vorhanden ist, wird enstandenes Xp nicht mehr verbraucht;
          #Biomassezerfall sorgt zusaetzlich fuer einen Anstieg von Xp
          #XsRW-Konzentration steigt an;
          #Biomasse wird reduziert (Zuwachs<Aufbau);
          #Zunahme an Zerfallsprodukten (u.a. Xp) reduziert sich durch abnehmende Xh-Konzentration
          
          dSsCSB[k]  = 0                    #Annahme: Ss ist AUFGEBRAUCHT und im dritten Atmungsniveau nicht mehr vorhanden
          dXs[k]     = 0                    #Annahme: Xs, der in der urspruenglichen Abwasserprobe war, ist AUFGEBRAUCHT und im dritten Atmungsniveau nicht mehr vorhanden
          
          dXh_g[k]   = Gesamt_5_list[[1]]$dcsb_endogen[k]*yH/(1-yH) #Annahme: mit Teilen von Xp wird Biomasse weiter aufgebaut (O2 Verbrauch durch Aufbau)
          dXs[k]     = dXh_g[k-1]*1/yH          #Xs wird infolge des Xh Aufbaus (im vorherigen Zeitschritt) neu gebildet
          dXh_b[k]   = Xh[k-1]*bh
          Xh[k]      = Xh[k-1]-dXh_b[k]+dXh_g[k]
          
          dXp[k]     = dXh_b[k]*fXp - dXh_g[k]*1/yH     # minus (Anteil an xs, der im Zuge der Biomassebildung verbraucht wird)
          dXip[k]    = dXh_b[k]*fXi
          #dXsRW[k]   = 0            #XS aus dem Rohabwasser ist vollstaendig abgebaut. uebrig gebliebener XS ist auf Zerfallsprodukte der Biomasse zurueckzufuehren
          #infolge der Nitrifiaktion (4.33mg L-1 O2 fuer 1mg L-1 NO3-N) kann rechnerisch kein Sauerstoff fuer den C-Abbau zur verfuegung stehen
          #Laboranalytisch sinkt der CSB weiter (geringe Abnahme)
          #infolge der Xp Zunahme steigt rechnerisch (Modell) die XsRW-Konzentration an
          dXsRW[k] = dXs[k] - dXp[k]
          Si[k]=Si[1]+((as.numeric(na.omit(Gesamt_5_list[[1]]$CSBmf))[length(na.omit(Gesamt_5_list[[1]]$CSBmf))]-Si[1])/nrow(Gesamt_5_list[[1]])*k)
        }
      }
      
      #Teilsummen der Kohlenstofffraktionen werden gebildet:
      Ssges=sum(dSsCSB); Xipges=sum(dXip); Xpges=sum(dXp); XsRWges=sum(dXsRW)
      
      #Kohlenstofffraktionen ueber den Versuchsablauf:
      Ss=Ssges-cumsum(dSsCSB); Xp=cumsum(dXp); XsRW=XsRWges-cumsum(dXsRW)
      Xip=cumsum(dXip); Xh = Xh ; Si=Si
      
      #Fuer den Vergleich zwischen modellierten und gemessenen Werten werden diese mit nicht-linearer Regression ermittelt
      csb_df    = data.frame(subset(cbind(Gesamt_5_list[[1]]$Minutenvektor,Gesamt_5_list[[1]]$CSB, Gesamt_5_list[[1]]$CSBmf, Gesamt_5_list[[1]]$BCOD), Gesamt_5_list[[1]]$CSB != "NA"))
      colnames(csb_df) = c("dauer", "csb", "csbmf", "bcod")
      CSB_tende          = csb_df$csb[nrow(csb_df)]
      
      ###############################################################_
      COD.recovery = (csb_df$csb[nrow(csb_df)] + sum(Gesamt_5_list[[1]]$dcsb_gesamt[1:(csb_df$dauer[nrow(csb_df)]*2)])) / csb_df$csb[1]
      COD.r = round(COD.recovery*100, digits = 2)
      ###############################################################_
      #----Kann Partikulaerer CSB-Anteil fraktioniert werden?
      
      if(any(!is.na(csb_df$bcod))){
        #es existieren die partikulaere Fraktionen: Xs, Xi,Xh
        Xs_t0 = csb_df$bcod[1] - Ss[1]  #1. Iterationsschritt # Anfangskonzentration Xs (nach Roeleveld et al. 2002)
        Xi_t0 = csb_df$csb[1] - Xs_t0 - Ss[1] - Si[1] - Xh[1] # Anfangskonzentration Xi (nach Roeleveld et al. 2002)
        #Entscheidung gegen Anfangsschaetzung (nach WERF 2003) -> nachzulesen M.Sc. SIMIC 
        Xi_ges = Xi_t0 + cumsum(dXip)
        Xs_tende_Xs_n_null = CSB_tende - Xi_ges[csb_df$dauer[nrow(csb_df)]*2] - Si[csb_df$dauer[nrow(csb_df)]*2] - Xh[csb_df$dauer[nrow(csb_df)]*2]
        
        XsRW = (XsRWges + Xs_tende_Xs_n_null) - cumsum(dXsRW)
        X_ges = XsRW + Xi_ges + Xh 
        Xs_ges = XsRW 
        Xs_ges[which(Xs_ges<0)]=0
        XsRW[which(XsRW<0)]=0
        CSBsum_Xs_neu = Ss + X_ges + Si # + Xp (ist bereits in Schleife beruecksichtigt)
        #XsRW: Menge an Xs die unter Beruecksichtigung der Neubildung (Xp) verbraucht wird.
        #Xs im Abwasser zum Zeitpunkt t_0 enthalten. Heterotrophe Biomasse (Xh) nicht vorhanden.
        #Unter dem Abbau von Ss direkt und/oder indirekt als hydrolytisches Produkt aus Xh-Zerfall gebildet.
        #Zerfallsprodukte von Xh sind abbaubar (Xp) und inert (Xip). Xs-Speicher wird durch den Zerfall von
        #Biomasse aufgefüllt.
        #Annahme: Xs ist am Ende des Versuches NICHT aufgebraucht (Versuch: BCOD nach XY Tagen abgebaut -> Xs_ges zum Versuchsende != 0)
        #NEIN: CSB-Konzentration zum Zeitpunkt t_Ende setzt sich zusammen aus Xip, Xp Xh und Xs
        #in der CSB-Konzentration zum Zeitpunkt t_Ende ist Ss aufgebraucht
        #um Xs zu berechnen muss von der CSB-Konz. die gebildete het. Biomasse (Xh) und deren Zerfallsprodukte Xp und Xip abgezogen werden
        
        #CSBsum: setzt sich zusammen aus den geloesten Fraktionen Si und Ss zusammen.
        #Aus der heterotrophen Biomasse, die gebildet wird und z.T. im Laufe des Versuches
        #anteilig in die Fraktionen Xip und Xp wieder zerfaellt. Xs (abbaubar) setzt sich
        #zunaechst (t=0) auschliessliche aus Xs_v also ohne Xi, spaeter (t_Ende) aus Xip, Xp
        #und xh (und Xs_v) zusammen
      }else{
        #es existiert ausschliesslich X als einzige partikulaere Fraktion
        Xi_ges = Xip
        Xs_tende_Xs_n_null = CSB_tende - Xi_ges[csb_df$dauer[nrow(csb_df)]*2] - Si[csb_df$dauer[nrow(csb_df)]*2] - Xh[csb_df$dauer[nrow(csb_df)]*2]
        XsRW = (XsRWges + Xs_tende_Xs_n_null) - cumsum(dXsRW)
        
        X_ges = XsRW + Xi_ges + Xh
        X_ges[which(X_ges<0)]=0
        Xs_ges=NA
        XsRW=NA  #Xi[1] kann nicht bestimmt werden. Xi[1] Teil der Xs-Konzentration  
        Xi_ges = NA #Xi[1]!=0 -> ohne fehlende BSB_5 bestimmung keine Angabe zu Startkonzentration
        CSBsum_Xs_neu = Ss + X_ges + Si # + Xp (ist bereits in Schleife beruecksichtigt)
      }  
      
      DF_CSB_Fraktionen = data.frame(cbind(CSBsum_Xs_neu, Ss,X_ges,Xs_ges, XsRW, Xh, Xp, Si, Xi_ges))
      colnames(DF_CSB_Fraktionen)=c("CSBges","Ss","X","Xs","XsRW", "Xh", "Xp", "Si", "Xi")         
      dim(DF_CSB_Fraktionen)
      
      #Zusammenfassung: Modellqualitaet, Konzentration einzelner Fraktionen (t_0 und t_Ende)
      Fraktionen_t_0 = DF_CSB_Fraktionen[1,]
      Fraktionen_t_n = DF_CSB_Fraktionen[nrow(DF_CSB_Fraktionen),]
      DF_Fraktionen = cbind(Fraktionen_t_0, Fraktionen_t_n)
      colnames(DF_Fraktionen)=c("t_0", "t_n")
      
      ##################################################################_
      #----Modell vs. integrierte Labor-Daten
      
      rlm1=lm(Gesamt_5_list[[1]]$CSB[1:Pos_Ss_Ende] ~ Gesamt_5_list[[1]]$Minutenvektor[1:Pos_Ss_Ende])
      rmse1=sqrt(mse(actual = na.omit(Gesamt_5_list[[1]]$CSB[1:Pos_Ss_Ende]), predicted = rlm1$fitted.values))
      rlm1_sum=summary(rlm1)
      rlm2=lm(Gesamt_5_list[[1]]$CSB[(Pos_Ss_Ende+1):Pos_Xs_Ende] ~ Gesamt_5_list[[1]]$Minutenvektor[(Pos_Ss_Ende+1):Pos_Xs_Ende])
      rmse2=sqrt(mse(actual = na.omit(Gesamt_5_list[[1]]$CSB[(Pos_Ss_Ende+1):Pos_Xs_Ende]), predicted = rlm2$fitted.values))
      rlm2_sum=summary(rlm2)
      rmse=c(rmse1,rmse2);korrelation=mean(c(rlm1_sum$r.squared,rlm2_sum$r.squared))
      if(Pos_Xs_Ende==Pos_Versuchsende){}else{
        rlm3=lm(Gesamt_5_list[[1]]$CSB[(Pos_Xs_Ende+1):Pos_Versuchsende] ~ Gesamt_5_list[[1]]$Minutenvektor[(Pos_Xs_Ende+1):Pos_Versuchsende])
        rmse3=sqrt(mse(actual = na.omit(Gesamt_5_list[[1]]$CSB[(Pos_Xs_Ende+1):Pos_Versuchsende]), predicted = rlm3$fitted.values))
        rmse=mean(c(rmse1,rmse2,rmse3)); rlm3_sum=summary(rlm3)
        korrelation=mean(c(rlm1_sum$r.squared,rlm2_sum$r.squared,rlm3_sum$r.squared))
      }
      rmse=mean(rmse)
      
      Pruefkriterien     = c(rmse, korrelation)
      Kalibrierparameter = c(Parameter_Y_h[n], Parameter_f_xi[o])
      
      kalibrierkombination = paste("Code",o,n,sep = "_")
      pruefparameter =  round(c(Kalibrierparameter,na.omit(Gesamt_5_list[[1]]$CSB)[1], na.omit(Gesamt_5_list[[1]]$CSB)[length(na.omit(Gesamt_5_list[[1]]$CSB))], sum(Gesamt_5_list[[1]]$dcsb_gesamt), COD.r,
                                as.numeric(as.character(Fraktionen_t_0)), as.numeric(as.character(Fraktionen_t_n)), Pruefkriterien), digits = 2)
      
      Zusammenfassung_pruefparameter = cbind(Zusammenfassung_pruefparameter, pruefparameter)
      Zusammenfassung_kalibrierkombination = c(Zusammenfassung_kalibrierkombination, kalibrierkombination)
      
      ##################################################################_
      ##################################################################_
      #----PLOT: ASM-Fraktionen
      minuten = (Gesamt_5_list[[1]]$Datime-Gesamt_5_list[[1]]$Datime[1]) / 60
      stunden = round(minuten/60, digits = 0)
      if(max(stunden)>25){
        x_aufteilung = stunden*60*10; x_beschriftung = stunden*10
      }else{
        x_aufteilung = stunden*60; x_beschriftung = stunden}
      plot_height = 9.375; plot_width = 12.5
      Link=paste("2_Abbildungen/5_Modell/Modell_alle/", Versuchstage[Tag] ,"_ASM1-Fraktionen_", kalibrierkombination,".pdf", sep="")
      pdf(file = Link, width = plot_width, height = plot_height)
      par(mar=c(6,8,4,4),mgp=c(5,2,0), mfrow=c(1,1), cex.axis=2.5, cex.lab=2.5, cex.main=3, xpd=FALSE)
      x = Gesamt_5_list[[1]]$Minutenvektor/60
      plot(x = x, y = DF_CSB_Fraktionen[,1], col="black", type = "l", ylab = "",lwd=6, xlab="",
           ylim = range(0,max(DF_CSB_Fraktionen, Gesamt_5_list[[1]]$CSB, na.rm = T)))
      grid(nx = NULL, ny = NULL, lwd = 3, col="gray50", lty = 2)
      lines(x = x, y = DF_CSB_Fraktionen[,1], col="black",lty=1, lwd=7)  # gesamt
      CSB_ges_alle = cbind(CSB_ges_alle, DF_CSB_Fraktionen[,1])
      #Kohlenstoff geloest
      lines(x = x, y = DF_CSB_Fraktionen[,2], col="tomato3",lty=1, lwd=5) #geloest abbaubar
      lines(x = x, y = DF_CSB_Fraktionen[,8], col="tomato3", lty=2,lwd=5)  # geloest inert
      #Kohlenstoff partikulaer
      if(any(!is.na(csb_df$bcod))){
        lines(x = x, y = DF_CSB_Fraktionen[,4], col="steelblue4", lty=1,lwd=5)      # Xs, der unter Beruecksichtigung der neugebildeten heterotrophen Biomasse verbraucht wird
        lines(x = x, y = DF_CSB_Fraktionen[,9], col= "steelblue4", lty=3,lwd=5)     # Xi aus Biomassezerfall
        points(x = x, y = Gesamt_5_list[[1]]$CSB,  type="p", pch = 21, lwd=2, cex = 3,col="gray80", bg="gray30")
        
        legend("topright", pch=c(NA,NA,NA,NA,NA,21),lty=c(1,1,2,1,3,NA),
               legend=c(expression(CSB[mod]),expression(S[S*","*mod]),expression(S[I*","*mod]),expression(X[S*","*mod]),expression(X[I*","*mod]),"CSB"), pt.bg = c(NA,NA,NA,NA,NA,"gray30"),
               col=c("black","tomato3","tomato3","steelblue4","steelblue4","gray80"),
               cex=2, inset=0.05, lwd=c(6,4,4,4,4,2), bg = alpha("white", 0.9), pt.cex = c(NA,NA,NA,NA,NA,3))
      }else{
        lines(x = x, y = DF_CSB_Fraktionen[,3], col="steelblue4", lty=1,lwd=5)      # X neugebildete heterotrophe Biomasseberuecksichtigt; keine Trennung von Xi und Xs moeglich, weil Xi[1] unbekannt
        points(x = x, y = Gesamt_5_list[[1]]$CSB,  type="p", pch = 21, lwd=2, cex = 3,col="gray80", bg="gray30")
        
        legend("topright", pch=c(NA,NA,NA,NA,21),lty=c(1,1,2,1,NA),
               legend=c(expression(CSB[mod]),expression(S[S*","*mod]),expression(S[I*","*mod]),expression(X[mod]),"CSB"), pt.bg = c(NA,NA,NA,NA,"gray30"),
               col=c("black","tomato3","tomato3","steelblue4","gray80"),
               cex=2, inset=0.05, lwd=c(6,4,4,4,2), bg = alpha("white", 0.9), pt.cex = c(NA,NA,NA,NA,3))
      }
      axis(1); axis(2)
      labNames="Konzentration"
      title(xlab = expression("Versuchszeit [h]"),ylab = substitute(paste(nn[], " [mg "* L^-1*"]"), list(nn=labNames[1])))
      dev.off()
    }
  }
  
  write.table(x = CSB_ges_alle, file = paste("3_Ausgabe/5_ASM_Kalibrierkombinationen_ALLE/CSB_ges_alle/",Versuchstage[Tag], "_CSB_ges_alle.txt",sep = ""), dec = ".",
              row.names = FALSE, col.names = FALSE, sep = "\t",quote = FALSE)
  
  Namensliste = c("Code", "Y_h", "f_xi", "CSB_t_0", "CSB_t_Ende", "SVR_CSB","COD.recovery",
                  "CSB_t_0_modell","Ss_t_0","X_t0","Xs_t0","XsRW_t_0","Xh_t_0","Xsp_t_0","Si_t_0","Xi_t_0",
                  "CSB_t_Ende_modell","Ss_t_Ende","X_t_Ende","Xs_t_Ende","XsRW_t_Ende","Xh_t_Ende","Xsp_t_Ende","Si_t_Ende","Xi_t_Ende",
                  "RMSE","Korrelation")
  
  Zusammenfassung_kalibrierkombinationen = data.frame(cbind(Zusammenfassung_kalibrierkombination, t(Zusammenfassung_pruefparameter)))
  colnames(Zusammenfassung_kalibrierkombinationen) = Namensliste
  
  write.table(x = Zusammenfassung_kalibrierkombinationen, file = paste("3_Ausgabe/5_ASM_Kalibrierkombinationen_ALLE/",Versuchstage[Tag], ".txt",sep = ""), dec = ".",
              row.names = FALSE, col.names = TRUE, sep = "\t",quote = FALSE)
  
  Dateiname_Ergebnis = paste("3_Ausgabe/Datenzusammenfassung/5/", Versuchstage[Tag], "_Respirationsversuch.txt", sep = "")
  
  Gesamt_5.1_list_name = colnames(Gesamt_5_list[[1]])[6]
  Gesamt_5.1_list     = Gesamt_5_list[[1]][6]
  
  Pos_in_Liste = which(Gesamt_5_list[[2]]$Datime == Gesamt_5_list[[1]]$Datime[1])
  
  MAT = matrix(data = NA, nrow = nrow(Gesamt_5_list[[2]]), ncol = ncol(Gesamt_5.1_list))
  
  Gesamt_5 = cbind(Gesamt_5_list[[2]][,1:5], MAT, Gesamt_5_list[[2]][,-1:-5])
  
  col.nr = seq(1,ncol(Gesamt_5.1_list),1) + 5
  row.nr = seq(1,nrow(Gesamt_5_list[[1]]),1) + Pos_in_Liste - 1
  Gesamt_5[row.nr, col.nr] = Gesamt_5.1_list
  col.name = c(colnames(Gesamt_5_list[[2]][,1:5]),Gesamt_5.1_list_name,colnames(Gesamt_5_list[[2]][,-1:-5]))
  colnames(Gesamt_5) = col.name
  
  write.table(x = Gesamt_5, file = Dateiname_Ergebnis, sep = "\t", col.names = TRUE, row.names = FALSE, dec = ".")
  
}
##############################################################


##############################################################_
##############################################################_
#---BESTE MODELL - KALIBRIERUNG

Tag=1
for(Tag in 9:length(Versuchstage)){
  Dateiname_Ergebnis = paste("3_Ausgabe/Datenzusammenfassung/5/", Versuchstage[Tag], "_Respirationsversuch.txt", sep = "")
  Gesamt_5  = read.table(file = Dateiname_Ergebnis, sep="\t", header = TRUE, dec = ".")
  Datime = strptime(Gesamt_5$Datime, format="%Y-%m-%d %H:%M:%S", tz="UTC")
  Gesamt_5 = data.frame(Datime, Gesamt_5[,-1])
  
  Gesamt_5_SVR = subset(Gesamt_5, Gesamt_5$SVR!=0)
  Gesamt_5_SVR = Gesamt_5_SVR[,1:21]
  
  Gesamt_6_list = list(Gesamt_5_SVR, Gesamt_5)
  
  #Suchen nach der besten Kombination an Kalibrierparameter
  ##############################################################_
  suchen    = NA # spezieller Suchbegriff (z.B. "3_3_Wentzel") oder NA
  Top_day = NULL; Top_char = NULL; Top = NULL
  Bearbeitete_Versuchstage = gsub(".txt", "", x = dir("3_Ausgabe/5_ASM_Kalibrierkombinationen_ALLE/", pattern = ".txt")) # gsub("_Belebtschlammmodell_Ergebnis.txt", "",x = )
  for (Nr in Bearbeitete_Versuchstage) {
    
    Dateiname  = paste("3_Ausgabe/5_ASM_Kalibrierkombinationen_ALLE/", Nr, ".txt", sep = "")
    Daten      = read.table(file = Dateiname, header = TRUE, dec = ".", sep = "\t")
    
    if(!is.na(suchen)){
      code     = paste("Code_", suchen, sep = "")
      code_pos = which(Daten$Modell_Code == code)
      Top        = rbind(Top,as.numeric(Daten_sort[1,-1]))
      Top_day    = c(Top_day, Nr)
      Top_char   = c(Top_char, as.character(Daten_sort[1,1]))
    }else{
      Daten_sort = Daten[order(Daten$RMSE, decreasing = FALSE),]
      Top_5      = Daten_sort[1:5,]
      Top        = rbind(Top,as.numeric(Daten_sort[1,-1]))
      Top_day    = c(Top_day, Nr)
      Top_char   = c(Top_char, as.character(Daten_sort[1,1]))
    }
  }
  topspalten=c(1:2, 3:6, 7:10, 14:15, 16:19, 23:24, 25:26)
  Top_Liste  = data.frame(Top_day, Top[,topspalten])
  colnames(Top_Liste) = c("Datum", colnames(Daten)[topspalten+1])
  Top_Liste_neu = cbind(Top_Liste[,1:7],
                        Top_Liste[,8], NA,Top_Liste[,9],NA,Top_Liste[,10],NA,Top_Liste[,11],NA,Top_Liste[,12],NA,Top_Liste[,13],NA,
                        Top_Liste[,14],NA,Top_Liste[,15],NA,Top_Liste[,16],NA,Top_Liste[,17],NA,Top_Liste[,18],NA,Top_Liste[,19],NA,
                        Top_Liste[,20:21], Top[,12])
  
  
  Namen=colnames(Top_Liste[,8:19])
  Listen_namen=c(colnames(Top_Liste[,1:7]),Namen[1], paste(Namen[1],"_prozent", sep = ""),Namen[2], paste(Namen[2],"_prozent", sep = ""),Namen[3], paste(Namen[3],"_prozent", sep = ""),Namen[4], paste(Namen[4],"_prozent", sep = ""),Namen[5], paste(Namen[5],"_prozent", sep = ""),Namen[6], paste(Namen[6],"_prozent", sep = ""),Namen[7], 
    paste(Namen[7],"_prozent", sep = ""),Namen[8], paste(Namen[8],"_prozent", sep = ""),Namen[9], paste(Namen[9],"_prozent", sep = ""),Namen[10], paste(Namen[10],"_prozent", sep = ""),Namen[11], paste(Namen[11],"_prozent", sep = ""),Namen[12], paste(Namen[12],"_prozent", sep = ""),
    colnames(Top_Liste[,20:21]), "Xh_t_0")
  colnames(Top_Liste_neu)=Listen_namen
  
  ##################################
  ii=1
  for (ii in 1:nrow(Top_Liste)) {
    for (i in seq(8,30,2)){
      if(i<20){#t=0
        Top_Liste_neu[ii,(i+1)] = 100/Top_Liste_neu[ii,8]*Top_Liste_neu[ii,i]
      }else{#t=Ende
        Top_Liste_neu[ii,(i+1)] = 100/Top_Liste_neu[ii,20]*Top_Liste_neu[ii,i]
      }
      
    }
  }
  
  Top_Liste_neu[8:33]=round(x = Top_Liste_neu[8:33], digits = 1)
  
  Zusatz=round(cbind((Top_Liste_neu$Ss_t_0 + Top_Liste_neu$Xs_t0),
                     (100/Top_Liste_neu$CSB_t_0_modell*(Top_Liste_neu$Ss_t_0 + Top_Liste_neu$Xs_t0)),
                     (Top_Liste_neu$Ss_t_Ende + Top_Liste_neu$Xs_t_Ende),
                     (100/Top_Liste_neu$CSB_t_Ende_modell*(Top_Liste_neu$Ss_t_Ende + Top_Liste_neu$Xs_t_Ende)),
                     (Top_Liste_neu$Ss_t_0 + Top_Liste_neu$Si_t_0), 
                     (100/Top_Liste_neu$CSB_t_0_modell*(Top_Liste_neu$Ss_t_0 + Top_Liste_neu$Si_t_0)),
                     (Top_Liste_neu$Ss_t_Ende + Top_Liste_neu$Si_t_Ende),
                     (100/Top_Liste_neu$CSB_t_Ende_modell*(Top_Liste_neu$Ss_t_Ende + Top_Liste_neu$Si_t_Ende))), digits = 1)
  colnames(Zusatz) = c("BCSB_t_0","BCSB_t_0_prozent","BCSB_t_Ende","BCSB_t_Ende_prozent", "CSBmf_t_0","CSBmf_t_0_prozent","CSBmf_t_Ende","CSBmf_t_Ende_prozent")
  
  Top_Liste_neu=cbind(Top_Liste_neu[,1:9], Zusatz, Top_Liste_neu[,-1:-9])
  
  
  if(!is.na(suchen)){filenamen = paste("3_Ausgabe/6_ASM_Kalibrierkombinationen_TOP/Versuch_",code,".txt", sep = "")}else{filenamen = paste("3_Ausgabe/6_ASM_Kalibrierkombination_TOP/Versuch_ALLE.txt", sep = "")}
  #.txt wird geschrieben
  write.table(x = Top_Liste_neu, file = filenamen, dec = ".", col.names = TRUE, sep = "\t", quote = FALSE)
  
  if(!is.na(suchen)){filenamen = paste("3_Ausgabe/6_ASM_Kalibrierkombinationen_TOP/Versuch_",code,"_Ergebnisse.txt", sep = "")}else{filenamen = paste("3_Ausgabe/6_ASM_Kalibrierkombination_TOP/Versuch_ALLE.txt", sep = "")}
  
  TOP = read.table(file = filenamen, header = TRUE, sep = "\t",dec = ".")
  #################################################################_
  #-#-#-# Berechnung CSB-Fraktionen mit Belebtschlamm Modell (Activated Sludge No.1)----
  #Heterotropher Yield Koeffizient (Wachstumsrate)
  #Kalibrierparameter
  tag = which(TOP$Datum == Nr)
  
  yH=TOP$Y_h[Tag]  # [g CSB / g CSB] Zellertragsrate aktiver heterotropher Biomasse
  bh        = 0.62/(60*24)                       # [h-1]
  
  ####Welche Zerfallsprodukte entstehen?
  #ASM1 betrachtet im Gegensatz zum ASM3 den Zerfall
  fXi = TOP$f_xi[Tag]  # [g CSB / g CSB] Anteil der Biomasse, der zu inert-partikulaeren Stoffen fuehrt
  fXp = 1 - fXi # [g CSB / g CSB] Anteil der Biomasse, der zu abbaubaren-partikulaeren Stoffen fuehrt (Henze 1992)
  
  ##############################
  #"2014-04-28 14:54:00", "2014-05-05 23:53:00", "2013-04-02 14:52:00, "2018-03-05 13:57:00","2018-02-13 12:41:00"
  Ss_Ende = strptime(c("2013-03-11 17:18:00","2013-04-02 19:58:00","2013-04-09 16:18:00","2014-04-08 16:08:00",
                       "2014-04-28 20:51:00","2014-05-05 20:02:00","2018-02-13 17:19:00","2018-03-05 17:05:00","2019-02-21 14:01:00",
                       "2019-02-25 13:24:00","2019-02-26 13:11:00", "2019-03-05 11:35:00", "2019-04-08 12:52:00","2019-04-10 15:03:00",
                       "2020-03-05 14:39:00"), format = "%Y-%m-%d %H:%M:%S", tz = "UTC")
  
  Xs_Ende = strptime(c("2013-03-13 16:12:00","2013-04-05 06:26:00","2013-04-11 17:14:00","2014-04-11 15:34:00",
                       "2014-04-30 15:20:00","2014-05-09 02:25:00","2018-02-17 23:56:00","2018-03-07 10:13:00","2019-02-21 14:17:00",
                       "2019-02-25 14:20:00","2019-02-26 13:59:00","2019-03-05 14:09:00","2019-04-09 07:58:00","2019-04-11 08:58:00",
                       "2020-03-06 10:34:00"), format = "%Y-%m-%d %H:%M:%S", tz = "UTC")
  Pos_Ss_Ende = which(Gesamt_6_list[[1]]$Datime == Ss_Ende[Tag])
  Pos_Xs_Ende = which(Gesamt_6_list[[1]]$Datime == Xs_Ende[Tag])
  if(is.integer0(Pos_Xs_Ende)){Pos_Xs_Ende = nrow(Gesamt_6_list[[1]])}else{}
  Pos_Versuchsende = nrow(Gesamt_6_list[[1]])
  
  #Nullvektoren mit Belegung des ersten Wertes
  Xh=NULL; dXh_b=NULL; dSsCSB=NULL; dXs=NULL; dXh_g=NULL; dXp=NULL; dXip=NULL; dXsRW=NULL;Si=NULL
  Xh[1]=TOP$Xh_t_0[tag] # Biomasse nach Wentzel et al. 1995 vorhanden
  dXh_b[1]=0; dSsCSB[1]=0; dXs[1]=0; dXh_g[1]=0; dXp[1]=0; dXip[1]=0; dXsRW[1]=0
  Si[1] = 0.9*as.numeric(na.omit(Gesamt_6_list[[1]]$CSBmf))[length(na.omit(Gesamt_6_list[[1]]$CSBmf))] #Faktor 0.9 (Roeleveld et al. 1998)
  ########################################
  
  k=2
  for (k in 2:length(Gesamt_6_list[[1]]$dcsb_gesamt)){
    if (k <= Pos_Ss_Ende){
      dSsCSB[k] = Gesamt_6_list[[1]]$dcsb_SS[k]/(1-yH)
      dXs[k]    = Gesamt_6_list[[1]]$dcsb_XS[k]/(1-yH)
      dXh_g[k]  = dSsCSB[k] * yH ; dXh_b[k] = Xh[k-1] * bh ; Xh[k] = Xh[k-1] - dXh_b[k] + dXh_g[k]
      
      dXp[k]     = dXh_b[k-1] * fXp; dXip[k] = dXh_b[k]   * fXi
      dXsRW[k]   = dXs[k] - dXp[k]#Xs-Speicher wird aus Zerfallsprodukten der HB teilweise aufgefüllt 
      if( dXp[k]<dXs[k] ){ dXp[k]=0 }else{ dXp[k]=dXp[k]-dXs[k] }
      Si[k]=Si[1]+((as.numeric(na.omit(Gesamt_6_list[[1]]$CSBmf))[length(na.omit(Gesamt_6_list[[1]]$CSBmf))]-Si[1])/length(Gesamt_6_list[[1]]$dcsb_gesamt)*k)
      
    }else if ( (k<Pos_Xs_Ende) & (k>Pos_Ss_Ende) ){
      dSsCSB[k] = 0; dXs[k] = Gesamt_6_list[[1]]$dcsb_XS[k]/(1-yH)
      dXh_g[k] = dXs[k]  * yH;  dXh_b[k] = Xh[k-1] * bh; Xh[k] = Xh[k-1] - dXh_b[k] + dXh_g[k]
      dXp[k] = dXh_b[k] * fXp; dXip[k]    = dXh_b[k] * fXi
      dXsRW[k] = dXs[k] - dXp[k]
      if(dXp[k]<dXs[k]){ dXp[k]=0 }else{ dXp[k]=dXp[k]-dXs[k] }
      Si[k]=Si[1]+((as.numeric(na.omit(Gesamt_6_list[[1]]$CSBmf))[length(na.omit(Gesamt_6_list[[1]]$CSBmf))]-Si[1])/length(Gesamt_6_list[[1]]$dcsb_gesamt)*k)
    }else{
      dSsCSB[k] = 0; dXs[k] = 0
      dXh_g[k] = Gesamt_6_list[[1]]$dcsb_endogen[k]*yH/(1-yH); dXs[k] = dXh_g[k-1]*1/yH ; dXh_b[k] = Xh[k-1]*bh
      Xh[k] = Xh[k-1]-dXh_b[k]+dXh_g[k]
      dXp[k]  = dXh_b[k]*fXp - dXh_g[k]*1/yH; dXip[k] = dXh_b[k]*fXi
      dXsRW[k] = dXs[k] - dXp[k]
      Si[k]=Si[1]+((as.numeric(na.omit(Gesamt_6_list[[1]]$CSBmf))[length(na.omit(Gesamt_6_list[[1]]$CSBmf))]-Si[1])/length(Gesamt_6_list[[1]]$dcsb_gesamt)*k)
    }
  }
  #Teilsummen der Kohlenstofffraktionen werden gebildet:
  Ssges=sum(dSsCSB); Xipges=sum(dXip); Xpges=sum(dXp); XsRWges=sum(dXsRW)
  
  #Kohlenstofffraktionen ueber den Versuchsablauf:
  Ss=Ssges-cumsum(dSsCSB); Xp=cumsum(dXp); XsRW=XsRWges-cumsum(dXsRW)
  Xip=cumsum(dXip); Xh = Xh ; Si=Si
  
  csb_df    = data.frame(subset(cbind(Gesamt_6_list[[1]]$Minutenvektor,Gesamt_6_list[[1]]$CSB, Gesamt_6_list[[1]]$CSBmf, Gesamt_6_list[[1]]$BCOD), Gesamt_6_list[[1]]$CSB != "NA"))
  DF_CSB_BSB = delete.na(DF = csb_df, n = 1) #Anzahl der NA in einer Reihe
  colnames(DF_CSB_BSB) = c("Minutenvektor", "CSB", "CSBmf", "BCOD")
  CSB_tende          = DF_CSB_BSB$CSB[nrow(DF_CSB_BSB)]
  
  dBCOD_mod   = (as.vector(Gesamt_6_list[[1]]$dcsb_gesamt) / (1-yH)) - dXh_g # Biomasse wird aufgebaut, die als biologisch abbaubar einzustufen ist (BCOD)
  BCOD_modges = sum(dBCOD_mod)
  
  
  if(any(!is.na(DF_CSB_BSB$BCOD))){
    #es existieren die partikulaeren Fraktionen: Xs, Xi,Xh
    BCOD_tende_Xs_n_null = na.omit(Gesamt_6_list[[1]]$BCOD)[length(na.omit(Gesamt_6_list[[1]]$BCOD))]
    BCOD_mod_n_null = (BCOD_modges+BCOD_tende_Xs_n_null)  - (cumsum(dBCOD_mod) )
    
    Xs_t0 = DF_CSB_BSB$BCOD[1] - Ss[1]  # Anfangskonzentration Xs (nach Roeleveld et al. 2002)
    Xi_t0 = DF_CSB_BSB$CSB[1] - Xs_t0 - Ss[1] - Si[1] - Xh[1] # Anfangskonzentration Xi (nach Roeleveld et al. 2002)
    #Entscheidung gegen Anfangsschaetzung (nach WERF 2003) -> nachzulesen M.Sc. SIMIC 
    Xi_ges = Xi_t0 + cumsum(dXip)
    Xs_tende_Xs_n_null = CSB_tende -  Xi_ges[DF_CSB_BSB$Minutenvektor[nrow(DF_CSB_BSB)]*2] - Si[DF_CSB_BSB$Minutenvektor[nrow(DF_CSB_BSB)]*2]-Xh[DF_CSB_BSB$Minutenvektor[nrow(DF_CSB_BSB)]*2]
    XsRW = (XsRWges + Xs_tende_Xs_n_null) - cumsum(dXsRW)
    
    X_ges = XsRW + Xi_ges + Xh
    Xs_ges = XsRW + Xh
    Xs_ges[which(Xs_ges<=0)]=0
    XsRW[which(XsRW==0)]=0
    
    CSBges = Ss + X_ges + Si 
  }else{
    #es existiert ausschliesslich X als einzige partikulaere Fraktion
    BCOD_mod_n_null = BCOD_modges - cumsum(dBCOD_mod)
    
    Xi_ges = Xip
    Xs_tende_Xs_n_null = CSB_tende - Xi_ges[DF_CSB_BSB$Minutenvektor[nrow(DF_CSB_BSB)]*2] - Si[DF_CSB_BSB$Minutenvektor[nrow(DF_CSB_BSB)]*2]- Xh[DF_CSB_BSB$Minutenvektor[nrow(DF_CSB_BSB)]*2]
    XsRW = (XsRWges + Xs_tende_Xs_n_null) - cumsum(dXsRW)
    
    X_ges = XsRW + Xi_ges + Xh
    X_ges[which(X_ges==0)]=0
    
    Xs_ges = NA
    XsRW=NA  #Xi[1] kann nicht bestimmt werden. Xi[1] Teil der Xs-Konzentration  
    Xi_ges = NA #Xi[1]!=0 -> ohne fehlende BSB_5 bestimmung keine Angabe zu Startkonzentration
    CSBges = Ss + X_ges + Si 
  }  
  
  DF_CSB_Fraktionen = data.frame(cbind(CSBges, Ss, X_ges, Xs_ges, XsRW, Xh, Xp, Si, Xi_ges))
  colnames(DF_CSB_Fraktionen)=c("CSBges", "Ss","X", "Xs", "XsRW", "Xh", "Xp", "Si", "Xi")         
  dim(DF_CSB_Fraktionen)
  
   
  xx = Gesamt_6_list[[1]]$Minutenvektor
  
  
  if(is.na(Top_Liste$Xs_t0[Tag])){vec=c(1,2,3,8); leg_text=c("SS","X","SI")
  }else{vec=c(1,2,4,8,9); leg_text=c("SS","XS","SI","XI")}
  
  
  Zeitvektor = seq(0,80*60,10*60)
  abschnitt=Zeitvektor/60; x_abschnitt=xx; DF_CSB_Fraktionen_abschnitt=DF_CSB_Fraktionen[,vec]
#
  
   
  
  
#   for (i in 2:length(Zeitvektor)){
#     for (k in 1:length(vec)) {
#       pos=which((xx >= Zeitvektor[i-1]) & (xx < Zeitvektor[i]))
#       x_abschnitt[pos] = abschnitt[i-1]
#       DF_CSB_Fraktionen_abschnitt[pos,k]=max(DF_CSB_Fraktionen_abschnitt[pos,k], na.rm = T)
#     }
#   }
#   DF=cbind(unique(DF_CSB_Fraktionen_abschnitt),unique(DF_CSB_Fraktionen_abschnitt[,-1]))
# 
#   for (i in 2:(ncol(DF_CSB_Fraktionen_abschnitt))) {
#     for (m in 1:nrow(DF)) {
#       DF[m,i+4]=(100/DF[m,1]*DF[m,i])
#     }
#   }
  # col=c("gray20","gray45","gray70","gray95")
  # zeit       = as.character(seq(0,nrow(DF)*10,10))
  # Fraktionen = colnames(DF_CSB_Fraktionen_abschnitt)[-1]
  # 
  # Daten <- as.matrix(t(DF[,2:5]))
  # 
  # bp = barplot(Daten, main = "", names.arg = zeit[1:nrow(DF)], xlab = "Versuchszeit [h]", ylab = expression("Konzentration [mg "* L^-1*"]" ), col = col)
  # 
  # Null_werte_NA = DF
  # Null_werte_NA[which(Null_werte_NA[,2]==0),2]=NA
  # 
  # text(bp, as.vector(Null_werte_NA[,2])/2,labels=paste(round(as.vector(DF[,2+4]), digits = 0),"%", sep = ""),cex=.8)
  # text(bp, as.vector(DF[,2]+(DF[,3]/2)),labels=paste(round(as.vector(DF[,3+4]), digits = 0),"%", sep = ""),cex=.8)
  # text(bp, as.vector(DF[,2]+DF[,3]+(DF[,4]/2)),labels=paste(round(as.vector(DF[,4+4]), digits = 0),"%", sep = ""),cex=.8)
  # text(bp, as.vector(DF[,2]+DF[,3]+DF[,4]+(DF[,5]/2)),labels=paste(round(as.vector(DF[,5+4]), digits = 0),"%", sep = ""),cex=.8)
  # legend("topright", Fraktionen, cex = 1.3, fill = col)
  # 
  
  
  if(is.na(DF_CSB_BSB$CSBmf[nrow(DF_CSB_BSB)])){csbmf_ende = DF_CSB_BSB$CSBmf[nrow(DF_CSB_BSB)-1]}else{csbmf_ende = DF_CSB_BSB$CSBmf[nrow(DF_CSB_BSB)]}
  Ges_gel = DF_CSB_Fraktionen$Ss + DF_CSB_Fraktionen$Si
  Si_pos = which(Gesamt_6_list[[1]]$Minutenvektor>330) # Was ist aelter als 5,5h
  
  ##################################################################_
  ##################################################################_
  #----PLOT: ASM physikalisch-chemischer Methode und stoechimetrischer Berechnung---
  
  minuten = (Gesamt_6_list[[1]]$Datime-Gesamt_6_list[[1]]$Datime[1])/60
  stunden = round(minuten/60, digits = 0)
  if(max(stunden)>25){
    x_aufteilung = stunden*60*10
    x_beschriftung = stunden*10
  }else{
    x_aufteilung = stunden*60
    x_beschriftung = stunden}
  
  plot.new()
  pdf(file = paste("2_Abbildungen/5_Modell/Modell_top/", as.character(TOP$Datum[Tag]) ,"_ASM_Fraktionen.pdf", sep=""), width = 20, height = 20)
  
  par(mar=c(5,6,4,2)+0.1, mfrow=c(4,2),oma = c(0, 0, 0, 2))
  #1. Abbildung COD
  max_range = (max(Gesamt_6_list[[1]]$CSB, DF_CSB_Fraktionen$CSBges,na.rm = T))*1.1
  min_range = 0
  plot(xx, DF_CSB_Fraktionen$CSBges, main = expression(CSB), ylab = "[mg L-1]",xaxt = "n", cex.axis=2.5, cex.lab=2.5, type = "l", lwd = 6,
       xlab="Versuchszeit [h]", cex.main=4, ylim = range(min_range, max_range))
  axis(1, at=x_aufteilung, labels=x_beschriftung, cex.axis=2.5,las=1,cex.lab=2.5,mgp=c(2.5, 1.3, 0))
  grid(nx = NA, ny = NULL, lwd = 2, col="gray50", lty = 2)
  abline(v = x_aufteilung , col="gray50", lty=2, lwd =2)
  lines(xx, DF_CSB_Fraktionen$CSBges, lwd = 6)
  legend("topright", pch=c(NA,16),lty=c(1,NA),legend=c("Modell","Laborwert"),col=c("black","red"),cex=2,
         inset=0.05, lwd=c(6), bg = alpha("white", 0.9), pt.cex = 3)
  points(x = Gesamt_6_list[[1]]$Minutenvektor, y = Gesamt_6_list[[1]]$CSB, pch=16,col="red", cex=3, lwd=2)
  #2. Abbildung BCSB
  max_range = (max(Gesamt_6_list[[1]]$BCOD, BCOD_mod_n_null, na.rm = T))*1.1
  min_range = 0
  plot(xx, BCOD_mod_n_null, main = "", ylab = "[mg L-1]", xaxt = "n",cex.axis=2.5, cex.lab=2.5, type = "l", lwd = 6,
       xlab="Versuchszeit [h]", ylim = range(min_range, max_range))
  if(any(!is.na(Gesamt_6_list[[1]]$BCOD))){title(main = expression(BCSB), cex.main = 4)}else{title(main = expression(BCSB*Konzentrationsabnahme), cex.main = 4)}
  axis(1, at=x_aufteilung, labels=x_beschriftung, cex.axis=2.5,las=1,cex.lab=2.5,mgp=c(2.5, 1.3, 0))
  grid(nx = NA, ny = NULL, lwd = 2, col="gray50", lty = 2)
  abline(v = x_aufteilung , col="gray50", lty=2, lwd =2)
  lines(xx, BCOD_mod_n_null, lwd = 6)
  legend("topright", pch=c(NA,16),lty=c(1,NA),legend=c("Modell","Laborwert"),col=c("black","red"),cex=2,
         inset=0.05, lwd=c(6), bg = alpha("white", 0.9), pt.cex = 3)
  points(Gesamt_6_list[[1]]$Minutenvektor,Gesamt_6_list[[1]]$BCOD, pch=16,col="red", cex=3, lwd=2)
  
  #3. Abbildung CSB_mf
  max_range = (max(Gesamt_6_list[[1]]$CSBmf, Ges_gel, na.rm = T))*1.1
  min_range = 0
  plot(xx, Ges_gel, main = expression(CSB[mf]), ylab = "[mg L-1]", xaxt = "n",cex.axis=2.5, cex.lab=2.5,type = "l", lwd = 6,
       xlab="Versuchszeit [h]", cex.main=4, ylim = range(min_range,max_range))
  axis(1, at=x_aufteilung, labels=x_beschriftung, cex.axis=2.5,las=1,cex.lab=2.5,mgp=c(2.5, 1.3, 0))
  grid(nx = NA, ny = NULL, lwd = 2, col="gray50", lty = 2)
  abline(v = x_aufteilung , col="gray50", lty=2, lwd =2)
  lines(xx, Ges_gel, lwd = 6)
  legend("topright", pch=c(NA,16),lty=c(1,NA),legend=c("Modell","Laborwert"),col=c("black","red"),cex=2,
         inset=0.05, lwd=c(6), bg = alpha("white", 0.9), pt.cex = 3)
  points(x=Gesamt_6_list[[1]]$Minutenvektor, y=Gesamt_6_list[[1]]$CSBmf, pch=16,col="red", cex=3, lwd=2)
  #4. Abbildung CODpart
  CODpart = DF_CSB_Fraktionen$X
  max_range = (max((Gesamt_6_list[[1]]$CSB-Gesamt_6_list[[1]]$CSBmf), CODpart, na.rm = T))*1.1
  min_range = 0
  plot(xx, CODpart, main = expression(CSB[part]), ylab = "[mg L-1]", xaxt = "n",cex.axis=2.5, cex.lab=2.5,type = "l", lwd = 6,
       xlab="Versuchszeit [h]", cex.main=4, ylim = range(min_range, max_range))
  axis(1, at=x_aufteilung, labels=x_beschriftung, cex.axis=2.5,las=1,cex.lab=2.5,mgp=c(2.5, 1.3, 0))
  grid(nx = NA, ny = NULL, lwd = 2, col="gray50", lty = 2)
  abline(v = x_aufteilung , col="gray50", lty=2, lwd =2)
  lines(xx, CODpart, lwd = 6)
  legend("topright", pch=c(NA,16),lty=c(1,NA),legend=c("Modell","Laborwert"),col=c("black","red"),cex=2,
         inset=0.05, lwd=c(6), bg = alpha("white", 0.9), pt.cex = 3)
  X_lab = (Gesamt_6_list[[1]]$CSB - Gesamt_6_list[[1]]$CSBmf)
  points(Gesamt_6_list[[1]]$Minutenvektor, y = X_lab, pch=16,col="red", cex=3, lwd=2)
  #5. Abbildung Ss
  max_range = (max(DF_CSB_BSB$CSBmf - csbmf_ende, DF_CSB_Fraktionen$Ss, na.rm = T))*1.1
  min_range = 0
  plot(xx, DF_CSB_Fraktionen$Ss, main = expression(S[S]), ylab = "[mg L-1]", xaxt = "n",cex.axis=2.5, cex.lab=2.5,type = "l", lwd = 6,
       xlab="Versuchszeit [h]", cex.main=4, ylim = range(min_range,max_range))
  axis(1, at=x_aufteilung, labels=x_beschriftung, cex.axis=2.5,las=1,cex.lab=2.5,mgp=c(2.5, 1.3, 0))
  grid(nx = NA, ny = NULL, lwd = 2, col="gray50", lty = 2)
  abline(v = x_aufteilung , col="gray50", lty=2, lwd =2)
  lines(xx, DF_CSB_Fraktionen$Ss, lwd = 6)
  legend("topright", pch=c(NA,16),lty=c(1,NA),legend=c("Modell","Laborwert"),col=c("black","red"),cex=2,
         inset=0.05, lwd=c(6), bg = alpha("white", 0.9), pt.cex = 3)
  Ss_lab = Gesamt_6_list[[1]]$CSBmf - csbmf_ende
  points(Gesamt_6_list[[1]]$Minutenvektor, y = Ss_lab, pch=16,col="red", cex=3, lwd=2)
  #6. Abbildung Xs
  if(any(!is.na(DF_CSB_Fraktionen$Xs))){ 
    max_range = (max(DF_CSB_Fraktionen$Xs, na.rm = T))*1.1
    min_range = 0
    plot(xx, DF_CSB_Fraktionen$Xs, main = expression(X[S]), ylab = "[mg L-1]", xaxt = "n",cex.axis=2.5, cex.lab=2.5,type = "l", lwd = 6,
         xlab="Versuchszeit [h]", cex.main=4, ylim = range(min_range, max_range))
    axis(1, at=x_aufteilung, labels=x_beschriftung, cex.axis=2.5,las=1,cex.lab=2.5,mgp=c(2.5, 1.3, 0))
    grid(nx = NA, ny = NULL, lwd = 2, col="gray50", lty = 2)
    abline(v = x_aufteilung , col="gray50", lty=2, lwd =2)
    lines(xx, DF_CSB_Fraktionen$Xs, lwd = 6)
    legend("topright", pch=c(NA,16),lty=c(1,NA),legend=c("Modell","Laborwert"),col=c("black","red"),cex=2,
           inset=0.05, lwd=c(6), bg = alpha("white", 0.9), pt.cex = 3)
  }else{
    plot(1, 1, main = expression(X[S]), ylab = "[mg L-1]", xaxt = "n",cex.axis=2.5, cex.lab=2.5,type = "l", lwd = 6,
         xlab="Versuchszeit [h]", cex.main=4)
    grid(nx = NA, ny = NULL, lwd = 2, col="gray50", lty = 2)
    abline(v = stunden*60 , col="gray50", lty=2, lwd =2)
  }
  Xs_lab = Gesamt_6_list[[1]]$BCOD - (Gesamt_6_list[[1]]$CSBmf - DF_CSB_Fraktionen$Si)
  points(Gesamt_6_list[[1]]$Minutenvektor, y = Xs_lab, pch=16,col="red", cex=3, lwd=2)
  #7. Abbildung Si
  max_range = (max(Gesamt_6_list[[1]]$CSBmf[Si_pos], DF_CSB_Fraktionen$Si, na.rm = T))*2
  min_range = 0
  plot(xx, DF_CSB_Fraktionen$Si, main = expression(S[I]), ylab = "[mg L-1]", xaxt = "n",cex.axis=2.5, cex.lab=2.5,type = "l", lwd = 6,
       xlab="Versuchszeit [h]", cex.main=4, ylim = range(min_range,max_range))
  axis(1, at=x_aufteilung, labels=x_beschriftung, cex.axis=2.5,las=1,cex.lab=2.5,mgp=c(2.5, 1.3, 0))
  grid(nx = NA, ny = NULL, lwd = 2, col="gray50", lty = 2)
  abline(v = x_aufteilung , col="gray50", lty=2, lwd =2)
  lines(xx, DF_CSB_Fraktionen$Si, lwd = 6)
  legend("topright", pch=c(NA,16),lty=c(1,NA),legend=c("Modell","Laborwert"),col=c("black","red"),cex=2,
         inset=0.05, lwd=c(6), bg = alpha("white", 0.9), pt.cex = 3)
  
  Si_lab = c(rep(NA,Si_pos[1]-1),Gesamt_6_list[[1]]$CSBmf[Si_pos])
  points(x=Gesamt_6_list[[1]]$Minutenvektor, y=Si_lab, pch=16,col="red", cex=3, lwd=2)
  #8. Abbildung Xi (Xi-Anteil zum Zeitpunkt t=0 ist Null)
  if(any(!is.na(DF_CSB_Fraktionen$Xi))){ 
    max_range = (max(DF_CSB_Fraktionen$Xi, na.rm = T))*2
    min_range = 0
    plot(xx, DF_CSB_Fraktionen$Xi, main = expression(X[I]), ylab = "[mg L-1]", xaxt = "n",cex.axis=2.5, cex.lab=2.5,type = "l", lwd = 6,
         xlab="Versuchszeit [h]", cex.main=4, ylim = range(min_range,max_range))
    axis(1, at=x_aufteilung, labels=x_beschriftung, cex.axis=2.5,las=1,cex.lab=2.5,mgp=c(2.5, 1.3, 0))
    grid(nx = NA, ny = NULL, lwd = 2, col="gray50", lty = 2)
    abline(v = x_aufteilung , col="gray50", lty=2, lwd =2)
    lines(xx, DF_CSB_Fraktionen$Xi, lwd = 6)
    legend("topright", pch=c(NA,16),lty=c(1,NA),legend=c("Modell","Laborwert"),col=c("black","red"),cex=2,
           inset=0.05, lwd=c(6), bg = alpha("white", 0.9), pt.cex = 3)
  }else{
    plot(1, 1, main = expression(X[I]), ylab = "[mg L-1]", xaxt = "n",cex.axis=2.5, cex.lab=2.5,type = "l", lwd = 6,
         xlab="Versuchszeit [h]", cex.main=4)
    grid(nx = NA, ny = NULL, lwd = 2, col="gray50", lty = 2)
    abline(v = stunden*60 , col="gray50", lty=2, lwd =2)
  }
  Xi_lab = Gesamt_6_list[[1]]$CSB - Gesamt_6_list[[1]]$CSBmf -  (Gesamt_6_list[[1]]$BCOD - (Gesamt_6_list[[1]]$CSBmf ))
  points(x = Gesamt_6_list[[1]]$Minutenvektor, y = Xi_lab, pch=16,col="red", cex=3, lwd=2)
  
  dev.off()
  #################################################################_
  #################################################################
  
  print(max(stunden))
  
  Gesamt_6 = Gesamt_5
  Dateiname_Ergebnis = paste("3_Ausgabe/Datenzusammenfassung/6/", Versuchstage[Tag], "_Respirationsversuch.txt", sep = "")
  write.table(x = Gesamt_6, file = Dateiname_Ergebnis, sep = "\t", col.names = TRUE, row.names = FALSE, dec = ".")
  
  
  head(DF_CSB_Fraktionen)
  DF_ASM       = cbind(xx, DF_CSB_Fraktionen$CSBges, Ges_gel, BCOD_mod_n_null, CODpart, DF_CSB_Fraktionen$Ss, 
                       DF_CSB_Fraktionen$Si,DF_CSB_Fraktionen$Xs, DF_CSB_Fraktionen$Xi)
  df.name=c("Dauer","CSB_mod","CSB_gel_mod","BSB_mod","CSBpart_mod",
            "Ss_mod","Si_mod","Xs_mod","Xi_mod")
  DF_ASM_pruef = cbind(xx,
                       DF_CSB_Fraktionen$CSBges, Gesamt_6_list[[1]]$CSB,
                       Ges_gel, Gesamt_6_list[[1]]$CSBmf,
                       BCOD_mod_n_null, Gesamt_6_list[[1]]$BCOD,
                       CODpart,              X_lab,
                       DF_CSB_Fraktionen$Ss, Ss_lab,
                       DF_CSB_Fraktionen$Si, NA,
                       DF_CSB_Fraktionen$Xs, Xs_lab,
                       DF_CSB_Fraktionen$Xi, Xi_lab)
  head(DF_ASM_pruef)
  
  #---DATENSELEKTION
  #Suche nach Abschnitten in den Modellergebnissen, in denen das Modellergebnis nur XX % Abweichung vom Laborwert haben.
  #Laborwert = Realwert: Zeitpunkt, indem beide werte vorliegen, gilt der Laborwert.
  #Laboranalyse besitzt 
  
  #CSB_mf zu Beginn des Versuches hat Unsicherheit; Empfehlung 0.1um membranfiltrieren
  ##CSB_mf wird ueberschaetzt
  Faktor = 10 #Messunsicherheit 20% (BSB_5), 10% (CSB)
  head(DF_ASM_pruef)
  inter = NULL;k=1
  for (k in 1:8) {
    i=k*2
    if(any(is.na(DF_ASM[,((k)+1)]))){}else{
      plot(DF_ASM[,1],DF_ASM[,((k)+1)], type = "l", lwd = 4, col = "red")
      
      werte = data.frame(subset(cbind(DF_ASM_pruef[,1], DF_ASM_pruef[,i:(i+1)]), DF_ASM_pruef[,(i+1)] != "NA"))
      if((nrow(werte)<2) | (Tag>8)){# | (Tag>8)
        if((Tag>8)){if(nrow(werte)==0){points(DF_ASM[,1],DF_ASM[,((k)+1)], cex = 3, col="green")}else{
          inter=NULL;ii=1
          for (ii in 1:nrow(werte)) {
            if((werte[ii,2]>=werte[ii,3]*((100-Faktor)/100)) & (werte[ii,2]<=werte[ii,3]*((100+Faktor)/100))){
              inter = c(inter, werte[ii,2])
            }else{inter = c(inter, werte[ii,3])}
          }
          DF_ASM[,((k)+1)] = NA
          Pos=werte[,1]*2; if(Pos[1]==0){Pos[1]=1}
          DF_ASM[Pos,((k)+1)] = inter
          points(DF_ASM[,1],DF_ASM[,((k)+1)], cex = 3, col="green", pch=16)}
        }else{lines(DF_ASM[,1],DF_ASM[,((k)+1)], col="green")}
      }else{
        for (ii in 1:nrow(werte)) {
          if((werte[ii,2]>=werte[ii,3]*((100-Faktor)/100)) & (werte[ii,2]<=werte[ii,3]*((100+Faktor)/100))){inter = c(inter, TRUE)}else{inter = c(inter, FALSE)}
        }
        Werte = cbind(werte,inter); inter = NULL;iii=1
        for (iii in 1:nrow(Werte)) {
          if(Werte[iii,4]==TRUE){
            if((iii==1) | (iii==nrow(Werte))){
              if((iii==1)){
                if(Werte[iii,1]==0){lines(x = DF_ASM[(Werte[iii,1]+1):(Werte[(iii+1),1]-1),1], y = DF_ASM[(Werte[iii,1]+1):(Werte[(iii+1),1]-1),((k)+1)], col="green", lwd=2)
                }else{lines(x = DF_ASM[0:(Werte[iii,1]-1),1], y = DF_ASM[0:(Werte[iii,1]-1),((k)+1)], col="green", lwd=4); lines(x = DF_ASM[(Werte[iii,1]+1):(Werte[(iii+1),1]-1),1], y = DF_ASM[(Werte[iii,1]+1):(Werte[(iii+1),1]-1),((k)+1)], col="green", lwd=4)}
              }else{lines(x = DF_ASM[(Werte[iii,1]+1):nrow(DF_ASM),1], y = DF_ASM[(Werte[iii,1]+1):nrow(DF_ASM),((k)+1)], col="green", lwd=4)}
            }else{lines(x = DF_ASM[(Werte[iii,1]+1):(Werte[(iii+1),1]-1),1], y = DF_ASM[(Werte[iii,1]+1):(Werte[(iii+1),1]-1),((k)+1)], col="green", lwd=4)}
          }else{
            if((iii==1) | (iii==nrow(Werte))){
              if((iii==1)){
                if(Werte[iii,1]==0){DF_ASM[(Werte[iii,1]+1):(Werte[(iii+1),1]-1),((k)+1)] = NA
                }else{DF_ASM[0:(Werte[iii,1]-1),((k)+1)]=NA; DF_ASM[(Werte[iii,1]+1):(Werte[(iii+1),1]-1),((k)+1)] = NA}
              }else{DF_ASM[(Werte[iii,1]+1):nrow(DF_ASM),((k)+1)] = NA}
            }else{DF_ASM[(Werte[iii,1]+1):(Werte[(iii+1),1]-1),((k)+1)] = NA}}
        }
        Pos=Werte[,1]*2; if(Pos[1]==0){Pos[1]=1}
        DF_ASM[Pos,((k)+1)] = Werte[,3]
        lines(DF_ASM[,1],DF_ASM[,((k)+1)], col="red")
        points(Werte[,1], Werte[,3], col="green", cex = 3, pch = 16)
      }}
    k=k+1
    print(k)
  }
  
  DF__ASM = data.frame(Gesamt_6_list[[1]]$Datime, DF_ASM)
  colnames(DF__ASM) = c("Datime","Dauer","CSB_mod", "CSBmf_mod","BCSB_mod","CSBpart_mod","Ss_mod","Si_mod","Xs_mod", "Xi_mod")
  Dateiname_Ergebnis = paste("3_Ausgabe/6_Fraktionen_TOP/",Versuchstage[Tag], "_Henze_Ergebnis.txt", sep = "")
  write.table(x = DF__ASM, file = Dateiname_Ergebnis, dec = ".",
              row.names = FALSE, col.names = TRUE, sep = "\t",quote = FALSE)
}



##################################################################
#---METADATEN
Tag=1
BIG=NULL
for(Tag in 1:length(Versuchstage)){
  Dateiname_Ergebnis = paste("3_Ausgabe/6_Fraktionen_TOP/",Versuchstage[Tag], "_Henze_Ergebnis.txt", sep = "")
  DF_Henze  = read.table(file = Dateiname_Ergebnis, sep="\t", header = TRUE, dec = ".")
  Datime = strptime(DF_Henze$Datime, format="%Y-%m-%d %H:%M:%S", tz="UTC")
  DF_Henze = data.frame(Datime, DF_Henze[,-1])
  
  Dateiname_Ergebnis = paste("3_Ausgabe/Datenzusammenfassung/6/",Versuchstage[Tag], "_Respirationsversuch.txt", sep = "")
  DF  = read.table(file = Dateiname_Ergebnis, sep="\t", header = TRUE, dec = ".")
  Datime = strptime(DF$Datime, format="%Y-%m-%d %H:%M:%S", tz="UTC")
  DF = data.frame(Datime, DF[,-1])
  DF = subset(DF, DF$SVR_CSB != "NA")
  
  if(Tag>8){
    labor = ReadLABOR(Datei =  "../../1_Daten/2_Optimierter Kohlenstoffabbau - Labor/Laboranalyse/Laboranalyse.xlsx",
                      Registerblatt = Versuchstage[Tag])
    LABOR = FindinLABOR(ReadLABOR_Output = labor, Kriterien = c("1_0", "roh"), Parameter=c("CSB"), XTS = FALSE,
                        Versuchstag = NA,spectrolyser=T)
    #spectro::lyser-Daten:
    Link_scan = paste("../../1_Daten/2_Optimierter Kohlenstoffabbau - Labor/1_Primaerdaten/", Versuchstage[Tag],"/UVVIS", sep = "")
    Datei_scan = dir(Link_scan)[grep(pattern = ".par", dir(Link_scan))]
    SCAN_par   = ReadSCANtoLIST(Datei = paste(Link_scan, Datei_scan, sep = "/"),
                                format="%Y.%m.%d  %X", dec = ",", XTS = FALSE)
    Datei_scan = dir(Link_scan)[grep(pattern = ".fp", dir(Link_scan))]
    SCAN_fp    = ReadSCANtoLIST(Datei = paste(Link_scan, Datei_scan, sep = "/"),
                                format="%Y.%m.%d  %X", dec = ",", XTS = FALSE)
    if(ncol(SCAN_fp[[2]])>215){SCAN_fp[[2]] = SCAN_fp[[2]][,1:215]} #NaN werden abgeschnitten}
    
    DF_SCAN_fp = FindLABinSCAN(FindinLABOR_Output = LABOR, ReadSCANtoLIST_Output = SCAN_fp, Parameter = "Fingerprint",
                               spectrolyser_Ausgabe = "Mittelwert", XTS = FALSE)
    DF_SCAN_par = FindLABinSCAN(FindinLABOR_Output = LABOR, ReadSCANtoLIST_Output = SCAN_par, Parameter = "FALSE",
                                spectrolyser_Ausgabe = "Mittelwert", XTS = FALSE)
    
    reihe=nrow(DF)
    DATEN = cbind(DF[1:reihe,1:7], DF[1:reihe,26:ncol(DF)], DF_Henze[1:reihe,-1:-2])
    DATEN[1:reihe,8:(ncol(DF)-18)]=NA
    
    pos = which(DF_SCAN_par[[1]]$CSB!=0)
    for (iiii in pos) {
      POS_SCAN = which(DF_SCAN_par[[1]]$Datime_Labor_start[iiii] == DF$Datime)
      DATEN[POS_SCAN,8:23] = DF_SCAN_par[[2]][iiii,-1:-2]
      DATEN[POS_SCAN,24:239] = DF_SCAN_fp[[2]][iiii,-1:-2]
    }
  }else{
    reihe=nrow(DF)
    DATEN = cbind(DF[1:reihe,1:7], DF[1:reihe,26:ncol(DF)], DF_Henze[1:reihe,-1:-2])
  }
  DATEN=cbind(rep(Tag,nrow(DATEN)),DATEN)
  BIG = rbind(BIG, DATEN)
  print(paste("Stand der Berechnung: Versuch ",Versuchstage[Tag]))
}

colnames(BIG)[1] = "Versuchsnr."


Dateiname_Ergebnis = paste("3_Ausgabe/Datenzusammenfassung/7/Gesamt.txt", sep = "")
 write.table(x = BIG, file = Dateiname_Ergebnis, dec = ".",
              row.names = FALSE, col.names = TRUE, sep = "\t",quote = FALSE)

##################################################################_
Dateiname_Ergebnis = paste("3_Ausgabe/Datenzusammenfassung/7/Gesamt.txt", sep = "")
DAT  = read.table(file = Dateiname_Ergebnis, sep="\t", header = TRUE, dec = ".")
Datime = strptime(DAT$Datime, format="%Y-%m-%d %H:%M:%S", tz="UTC")
DAT = data.frame(Datime, DAT[,3], DAT[,1], DAT[,-1:-3])

#Einflussnahme Metadaten:
Bearbeiter = c("Weber","Weber","Weber","Paul","Paul","Paul","Linden","Linden","Wittwer","Wittwer", 
               "Wittwer","Wittwer","Wittwer","Wittwer","Weber")
Seriennummer = c("SN_11280256","SN_11280256","SN_11280256","SN_11280217","SN_11280217","SN_11280217",
                 "SN_11280255","SN_11280255","SN_11280255","SN_09210165","SN_09210165","SN_11280255",
                 "SN_14350400","SN_14350400","SN_11280255")
Messstandort = c("Kaditz","Kaditz","Kaditz","MS1","MS1","MS1","MS1","MS1","MS1","MS1","MS1","MS1","MS1","MS1","MS1")
Bearbeiter.vec.ges=NULL;Seriennummer.vec.ges=NULL;Messstandort.vec.ges=NULL
for (i in 1:15) {
  pos=which(DAT$DAT...1. == i)
  Bearbeiter.vec = rep(Bearbeiter[i],length(pos));Seriennummer.vec = rep(Seriennummer[i],length(pos));Messstandort.vec = rep(Messstandort[i],length(pos))
  Bearbeiter.vec.ges = c(Bearbeiter.vec.ges, Bearbeiter.vec);Seriennummer.vec.ges = c(Seriennummer.vec.ges, Seriennummer.vec);Messstandort.vec.ges = c(Messstandort.vec.ges, Messstandort.vec)
}

DAT=cbind(DAT[,1:3], Bearbeiter.vec.ges, Seriennummer.vec.ges, Messstandort.vec.ges, DAT[,-1:-3])
colnames(DAT)[1:6] = c("Datime", "Minutenvektor", "Versuchsnummer","Bearbeiter","Seriennummer","Messstandort")

Dateiname_Ergebnis1 = paste("3_Ausgabe/Datenzusammenfassung/8/Gesamt.txt", sep = "")
Dateiname_Ergebnis2 = paste("../03_Regressionsmodell/1_Daten/Gesamt.txt", sep = "")
write.table(x = DAT, file = Dateiname_Ergebnis1, dec = ".",
            row.names = FALSE, col.names = TRUE, sep = "\t",quote = FALSE)
write.table(x = DAT, file = Dateiname_Ergebnis2, dec = ".",
            row.names = FALSE, col.names = TRUE, sep = "\t",quote = FALSE)


##############################


Dateiname_Ergebnis = paste("3_Ausgabe/Datenzusammenfassung/8/Gesamt.txt", sep = "")
DAT  = read.table(file = Dateiname_Ergebnis, sep="\t", header = TRUE, dec = ".")
Datime = strptime(DAT$Datime, format="%Y-%m-%d %H:%M:%S", tz="UTC")
DAT = data.frame(Datime, DAT[,-1])
Ss_Ende = strptime(c("2013-03-11 17:18:00","2013-04-02 19:58:00","2013-04-09 16:18:00","2014-04-08 16:08:00",
                     "2014-04-28 20:51:00","2014-05-05 20:02:00","2018-02-13 17:19:00","2018-03-05 17:05:00","2019-02-21 14:01:00",
                     "2019-02-25 13:24:00","2019-02-26 13:11:00", "2019-03-05 11:35:00", "2019-04-08 12:52:00","2019-04-10 15:03:00",
                     "2020-03-05 14:39:00"), format = "%Y-%m-%d %H:%M:%S", tz = "UTC")

Xs_Ende = strptime(c("2013-03-13 16:12:00","2013-04-05 06:26:00","2013-04-11 17:14:00","2014-04-11 15:34:00",
                     "2014-04-30 15:20:00","2014-05-09 02:25:00","2018-02-17 23:56:00","2018-03-07 10:13:00","2019-02-21 14:17:00",
                     "2019-02-25 14:20:00","2019-02-26 13:59:00","2019-03-05 14:09:00","2019-04-09 07:58:00","2019-04-11 08:58:00",
                     "2020-03-06 10:34:00"), format = "%Y-%m-%d %H:%M:%S", tz = "UTC")

mat__ss=NULL;mat__xs=NULL;frak_col = seq(239,246,1)
for (i in frak_col) {
  vek_num_ss=NULL; vek_num_xs=NULL;ii=1
  for (ii in 1:15) {
    #Welche und wieviele Daten gehören zu dem jeweiligen Respirationsabschnitt?
    pos_versuch = which(DAT$Versuchsnummer == ii)
    pos_ss = which(DAT$Datime[pos_versuch] < Ss_Ende[ii])
    pos_xs = which(DAT$Datime[pos_versuch] > Ss_Ende[ii])
    vek_laenge_ss = length(DAT$Datime[pos_versuch][pos_ss]); vek_laenge_xs = length(DAT$Datime[pos_versuch][pos_xs])
    
    vek_na_ss = which(is.na(DAT[pos_versuch,i][pos_ss]))
    vek_na_xs = which(is.na(DAT[pos_versuch,i][pos_xs]))
    vek_na_ss_num = which(!is.na(DAT[pos_versuch,i][pos_ss]))
    vek_na_xs_num = which(!is.na(DAT[pos_versuch,i][pos_xs]))
    
    #Wieviel Prozent der Daten sind nicht NA und stehen für eine Auswertung zutr Verfügung?
    if(is.integer0(vek_na_ss)){Anteil_NA_ss=0
    }else{
      if((ii>8) | (is.integer0(vek_na_ss))){
        Anteil_NA_ss = 100/length(na.omit(DAT[pos_versuch,8][pos_ss]))*(length(na.omit(DAT[pos_versuch,8][pos_ss]))-length(vek_na_ss_num))
      }else{Anteil_NA_ss = 100/vek_laenge_ss*length(vek_na_ss)
      }
    } 
    
    if(is.integer0(vek_na_xs)){Anteil_NA_xs=0
    }else{
      if((ii>8) | (is.integer0(vek_na_xs))){
        Anteil_NA_xs = 100/length(na.omit(DAT[pos_versuch,8][pos_xs]))*(length(na.omit(DAT[pos_versuch,8][pos_xs]))-length(vek_na_xs_num))
      }else{Anteil_NA_xs = 100/vek_laenge_xs*length(vek_na_xs)
      }
    } 
    Anteil_num_ss = 100-Anteil_NA_ss; Anteil_num_xs = 100-Anteil_NA_xs
    
    vek_num_ss = c(vek_num_ss,Anteil_num_ss); 
    vek_num_xs = c(vek_num_xs,Anteil_num_xs)
    ii=ii+1
  }
  mat__ss=cbind(mat__ss, vek_num_ss); mat__xs=cbind(mat__xs,vek_num_xs)
}
mat__ss=round(mat__ss,digits = 2); mat__xs=round(mat__xs,digits = 2)
colnames(mat__ss) = colnames(DAT)[frak_col]; colnames(mat__xs) = colnames(DAT)[frak_col]
LIST_verfuegbar = list(mat__ss, mat__xs)

boxplot(mat__xs)

#colMeans(mat__xs)
