# Kalibrierfunktionen mit Partcial Least Squares Regression (PLS-R) 
# Beschreibung: PLS-R nach dem Ansatz von Torres 2008. Messreihen aus Respirationsversuchen 
#                  
# Ergebnis:         - Kalibrierfunktionen als *.txt 
#                   - Faktorladungen als *.txt 
#                   - Modellpraemissen, Regressionsbewertung als *.txt
#                   - Plots: o Korrelation zw. PLS-R Ergebnis (IST) und Referenz (SOLL) 
#                            o Faktorladungen 
#                            o Bootstarpping fuer Konfidenzintervall

###############################################################_
#---BIBLIOTHEK - Funktionen ----
source(file = "../FUNKTIONEN.R")

#----Lokale Funktionen ----
NA_in_0     = function(x)      {POS_NA = which(is.na(x)==TRUE); x[POS_NA] = 0; return(x)} #Umwandlung von NA in 0
NA_all      = function(x,Spalte)      {if(is.vector(x)){NA_TRUE=sum(is.na(x));length(x)==NA_TRUE}else{NA_TRUE=sum(is.na(x[,Spalte])); length(x[,Spalte])==NA_TRUE}}
euc.dist    = function(x1, x2) {sqrt(sum((x1 - x2) ^ 2))} #Berechnung der euklidischen Distanz
is.integer0 = function(x)      {is.integer(x) && length(x) == 0L} #Pruefung ob Variable ein integer ist TRUE/FALSE
delete.na   = function(DF, n=0) {DF[rowSums(is.na(DF)) <= n,]} #Loeschen von Zeilen mit definierter Anzahl an NA?s

###############################################################_
#----BIBLIOTHEK - package  ----
library(ISI.Toolbox);library(zoo);library(dygraphs);library(MASS);library(XLConnect);library(matrixStats)
library(ggplot2);library(scales);library(Metrics);library(wesanderson);library(gridExtra);library(grid);
library(pls);library(cluster);library(arm);library(lmtest);library(car)

###############################################################_
#----Datengrundlage wird eingelesen ----
Versuchstage = c("2013-03-11","2013-04-02","2013-04-09","2014-04-08","2014-04-28","2014-05-05","2019-02-21","2018-02-13",
                 "2018-03-05","2019-02-25","2019-02-26","2019-03-05","2019-04-08","2019-04-10","2020-03-05")
DAT    = read.table(file = "1_Daten/Gesamt.txt", sep="\t", header = TRUE, dec = ".")
Datime = strptime(DAT$Datime, format="%Y-%m-%d %H:%M:%S", tz="UTC")
DAT    = data.frame(Datime, DAT[,-1])
head(DAT)
###############################################################_
#----Selektion aus Datengrundlage ----
Namensvektor = c("CSB","CSBmf","BSB5","BCSB","CSB_mod",
                 "CSBmf_mod","BCSB_mod","CSBpart_mod","Ss_mod","Si_mod",
                 "Xs_mod","Xi_mod")#9 10 11 12

Name = 10
summary_mat = matrix(data = NA, nrow = length(Namensvektor), ncol = 2)
colnames(summary_mat) = c("R2_lokal", "RMSE_lokal")
rownames(summary_mat) = Namensvektor


#for (Name in 2:length(Namensvektor)) {
  #Wieviele Daten aus sollen aus den Abschnitten bezogen werden?
  DPproIntervall=15
  
  ##### Welche Werte werden fuer die Kalibrierung und Valdierung verwendet?
  Ordner = "3_Ausgabe/01_PLS-R/"
  File1   = paste("DF_select_",DPproIntervall,"_",Namensvektor[Name],".txt",sep = "")
  if(is.integer0(which(dir(Ordner) == File1))){
    DAT.select = dat.select(Daten = DAT,Selektparameter = c(Namensvektor[Name],"wl_435nm"), Anz.pro.Zeitabschnitt = DPproIntervall,
                             Zeitvektor = c(1,2,3,6,12,24,48,72))
    write.table(DAT.select, file = paste(Ordner, File1, sep = ""), sep="\t", row.names = TRUE)
    }else{
      DAT.select = read.table(file = paste(Ordner, File1, sep = ""), sep="\t", header = TRUE)
    }
 
  dim(DAT.select)
  #Auswahl des Regressanden
  #Auswahl und ggf. Eingrenzung der Regressoren
  #   Wellenlaengen aus Datengrundlage obere_wl = 200, untere_wl = 732.5
  wl.select = wl_auswahl(Daten = DAT.select, Datenauswahl = "wl-bereich", obere_wl = 220, untere_wl = 732.5)
  x        = wl.select[[1]]
  
  #Loeschen von Werten, die Null betragen 
  y.pos      = which(colnames(DAT.select)==Namensvektor[Name])
  y          = DAT.select[,y.pos]   #Regressand: Parameter
  y.null.pos = which(y > 0)
  Y          = y[y.null.pos]
  X          = x[y.null.pos,]
  
  ##### Welche Werte werden fuer die Kalibrierung und Valdierung verwendet?
    File   = paste("DF_KalVal_",DPproIntervall,"_",Namensvektor[Name],".txt",sep = "")
    if(is.integer0(which(dir(Ordner) == File))){
      kal_row = sample(x = seq(1, nrow(X),1), size = round(x = (nrow(X)*(2/3)), digits = 0), replace = FALSE)
      row     = seq(1, nrow(X),1)
      val_row = setdiff(row, kal_row)
      datenaufteilung = matrix(data = NA, nrow = length(kal_row), ncol = 2)
      datenaufteilung[,1]=kal_row; datenaufteilung[1:length(val_row),2]=val_row
      colnames(datenaufteilung) = c("Kalibrierung", "Valdierung")
      write.table(datenaufteilung, file = paste(Ordner, File, sep = ""), sep="\t", row.names = TRUE)
    }else{
      kal_val = read.table(file = paste(Ordner, File, sep = ""), sep="\t", header = TRUE)
      kal_row = kal_val$Kalibrierung
      val_row = kal_val$Valdierung
    }
    
    #Aufteilung in Kalibrierungs un Validierungsdatensatz
    X_kal   = X[kal_row,]; X_val = X[val_row,]; y_kal = y[kal_row]; y_val = y[val_row]

  ###############################################################
  #---- PLS (nach forward-Prinzip) ---- 
  
  #Vereinfachung durch Annahmen:
  
  # 1.Ansatz: 
  #     Mit dem One-Sigma Test wird auf der Grundlage aller WL die notwendige Anzahl latenter Vektoren ermittelt.
  #     Mit dem forward Ansatz von Torres (2008), bei dem die Anzahl an WL stetig gesteigert wird, 
  #     werden die WL-Kombinationen ermittelt, mit denen die besten Modellvorhersagen erzielt werden. 
  #     Die Lösung mit dem geringsten PRESS ist die Vorzugsvariante.  
  # fkt = plsr(Y ~ as.matrix(X), ncomp = 100, validation ="CV")
  # ncomp.onesigma = selectNcomp(fkt, method = "onesigma", plot = TRUE)
  # N.VL.max = ncomp.onesigma  # Anzahl latenter Vektoren/Faktoren
  
  # 2.Ansatz: 
  # Die Eigenschaft der Lichtadsorbtion wird im kommunalem Abwasser maßgeblich von sechs Komponenten beeinflusst -
  # Nitrat, Nitrit, gelöste Organik, anionische Tenside, Kolloide (Schwebstoffe), TSS (Thomas et al. 2008).
  # Jeder dieser Kompenenten verfuegt ueber eine wellenlängenspezifische Lichtabsorbtion, die Anteile an der 
  # Gesamtexktinktion besitzen (Thomas et al. 2008). Bezugnehmend auf den physikalisch-mathematischen Ansatz der 
  # Dekonvolution von Superpositionphänomenen in der Spektrophotometrie, bei der die Extinktion komponentenspezfisch 
  # separiert und näherungsweise bestimmt werden, wird innerhalb des PLS-Ansatzes die maximale Anzahl latenter 
  # Vektoren auf sechs begrenzt. Auf diese Weise wird die angestrebte Vereinfachung der Methode umgesetzt. 
  N.VL.max = 6  # Anzahl latenter Vektoren/Faktoren
  
  N.VI.max = 20             # Maximalzahl unabhaengige Variablen/Wellenlaengen (WL)
  
  # SUCHE NACH DER BESTEN KOMBINATION AUS ANZAHL AN WELLENLAENGEN UND FAKTOREN 
  
  ## Herausfinden der optimalen Zahl von N.VL und N.VI
  # nach der Methode von Torres und Bertrand-Krajewski (2008)
  
  XE.tri = NULL; r_quad = NULL; reihenf = NULL
  
  for (ii in 1:ncol(X_kal)) r_quad[ii] = cor(y_kal,X_kal[,ii])^2
  reihenf[1] = order(r_quad, decreasing=TRUE)[1] # Auswahl der WL mit bester Korrelation 
  mat.RMSEP  = matrix(rep(NA, N.VI.max*N.VL.max), nrow=N.VL.max,ncol=N.VI.max)
  mat.PRESS  = matrix(rep(NA, N.VI.max*N.VL.max), nrow=N.VL.max,ncol=N.VI.max)
  #PRESS:= Predictive Residual Sum of Squares; Summe der Fehlerquadrate von Schätzwert
  #Summe der Abweichungsquadrate aus einer Kreuzvalidierung
  #RMSEP:= Root Mean Squares Error of Prediction sqrt((PRESS/Anzahl an Proben)); wurzel des mittleren Fehlerquadrate von Schätzwert 
  mat.cor    = matrix(rep(NA, N.VI.max*N.VL.max), nrow=N.VL.max,ncol=N.VI.max)
  for (i in (1:N.VI.max)){
    if (i != 1){
      X.red = X_kal[,-reihenf] # Reduktion der Datengrundlage um die  bereits ausgewaehlte Spalte, damit Spalte nicht doppelt ausgewaehlt werden kann
      cor.y.X.red = cor(fkt$residuals[,1,N.VL], X.red) # Mit welcher Spalte der reduzierten Datengrundlage koennen die Residuen des vorhergehenden Schritts am besten beschrieben werden? Korrelation zwischen Residuen und X1...8
      next.col      = colnames(X.red)[which.max(abs(cor.y.X.red))]
      next.col.name = which(colnames(X_kal)==next.col)
      reihenf       = c(reihenf, next.col.name) # ausgewaehlte Spalten aus der gesamten Datengrundlage
    }
    XE.tri    = X_kal[,reihenf]
    N.VL      = min(i,N.VL.max)
    fkt       = plsr(y_kal ~ as.matrix(XE.tri), ncomp=N.VL, validation ="CV", jackknife = TRUE)
    rmsep.fkt = RMSEP(fkt)
    
    for (ii in (1:N.VL)){
      mat.RMSEP[ii,i] = rmsep.fkt$val[1,,ii+1]
      mat.PRESS[ii,i] = fkt$validation$PRESS[1,ii]
      mat.cor[ii,i]   = cor(fkt$fitted.values[,1,ii],y_kal)
    }
  }
  # Suche nach der besten Lösung
  pos.minPRESS = which(mat.PRESS == min(mat.PRESS, na.rm=TRUE), arr.ind = TRUE)
  pos.minRMSEP = which(mat.RMSEP == min(mat.RMSEP, na.rm=TRUE), arr.ind = TRUE)
  best.N.VI    = pos.minPRESS[1,2]
  best.N.VL    = pos.minPRESS[1,1]
  print(paste("Mit ", best.N.VI, " Wellenlaengen und ", best.N.VL, " Faktoren wird mit ",
              round(mat.RMSEP[best.N.VL,best.N.VI],digits = 3)," der kleinste RMSE erzeugt", sep = ""))
  
  ###############################################################
  #----PLS-R mit einer AUSGEWAEHLTEN Wellenlaengen- und Faktorenanzahl----
  # Wellenlaengen werden aus sortierter Matrix ausgewaehlt 
  reihenf  = as.vector(as.matrix(reihenf))
  
  N.VI_wahl = best.N.VI       # Anzahl unabhaengige Variablen/Wellenlaengen (WL)
  N.VL_wahl = best.N.VL       # Anzahl latente Vektoren/Faktoren
  # HINWEIS: beide Groessen selbst waehlen oder lt. PRESS von oben uebernehmen
  
  wel.vi = sort(reihenf[1:N.VI_wahl]) # WL, die in Kombination signifikant sind; Reihenfolge wird mit sort() augeloest
  XE.tri = X_kal[,wel.vi]             # reduzierte Spektraldaten fuer Kalibrierung 
  N.VL   = min(N.VL_wahl, N.VL.max, N.VI_wahl) 
  # bei selbst gewaehltem N.VI_wahl kann die maximale moegliche Anzahl an Faktoren 
  # und die zur Verfuegung stehende Anzahl an WL nicht ueberschritten werden; 
  
  # Regressionsfunktion 
  fkt = plsr(y_kal ~ as.matrix(XE.tri), ncomp = N.VL_wahl, validation ="CV", jackknife = TRUE)
  
  ###############################################################
  #----PLS-R Auswertung und graphische Darstellellung ----
  ###############################################################
  # Korrelationsplot
  y_kal_fit  = predict(object = fkt, newdata = as.matrix(XE.tri), ncomp = N.VL_wahl)
  XE.tri_val = as.matrix(X_val[,wel.vi])             # reduzierte Spektraldaten fuer Validierung 
  y_val_fit  = predict(object = fkt, newdata = XE.tri_val, ncomp = N.VL_wahl)
  
  #Kalibrierung
  RMSE_kal   = round(Metrics::rmse(actual = y_kal, predicted = y_kal_fit), digits = 3)
  R2_kal     = round(cor(x = y_kal_fit, y = y_kal)^2, digits = 3)
  Anz_sample = dim(XE.tri)[1]
  R2_adj_kal = round(1-(1-R2_kal)*(Anz_sample-1)/(Anz_sample-N.VL_wahl-1), digits = 3)
  #Validierung
  DF_Val=na.omit(cbind(y_val_fit,y_val))
  R2_val         = round(cor(x = DF_Val[,1], y = DF_Val[,2])^2, digits = 3)
  RMSE_val       = round(Metrics::rmse(actual = DF_Val[,1], predicted = DF_Val[,2]), digits = 3)
  Anz_sample = dim(XE.tri_val)[1]
  R2_adj_val     = round(1-(1-R2_val)*(Anz_sample-1)/(Anz_sample-N.VL_wahl-1), digits = 3)
  
  print(paste("Ergebnisse der Kalibrierung: "," R2=", R2_kal, " R2_adj=",R2_adj_kal," RMSE=", RMSE_kal))
  print(paste("Ergebnisse der Valdierung: "," R2=", R2_val, " R2_adj=",R2_adj_val," RMSE=", RMSE_val))
  
  Speicher.Link = paste("2_Abbildungen/01_PLS-R/1 Korrelation - PLOT/Kor_PLS-R-Kal_",Namensvektor[Name],".pdf", sep = "")
  pdf(file = Speicher.Link, width = 12.5, height = 9.375)
  col_vec = wes_palette("Zissou1", 2, type = "continuous")
  par(mar=c(6,8,4,4),mgp=c(5,2,0), mfrow=c(1,1), cex.axis=2.5, cex.lab=2.5, cex.main=3, xpd=FALSE)
  x = y_kal_fit; y = y_kal
  plot(x = x, y = y, xlim = range(0, max(y_kal_fit,y_val_fit,y_kal,y_val,na.rm = T)), xlab ="", ylab="" ,
       ylim = range(0, max(y_kal_fit,y_val_fit,y_kal,y_val,na.rm = T)), pch = 16, lwd = 6)
  axis(1); axis(2)
  grid(nx = NULL, ny = NULL, lwd = 2, col="gray50", lty = 2) 
  abline(0,1, col="gray50", lty=1, lwd =2)
  points(x = x, y = y, pch=15, col = col_vec[1], lwd=2, cex = 3)
  points(x = y_val_fit, y = y_val, pch=16, col = col_vec[2], lwd=2, cex = 3)
  legend("topleft", legend = c("Trainingsset", "Testset"), col = col_vec, 
         inset = 0.025, bg = scales::alpha("white", 0.8), cex=2, pch = c(15, 16), pt.cex = 3)
  legend('bottomright', ncol = 2, bg = scales::alpha("white", 0.8), inset = 0.025, cex = 1.75, 
         title = "Ergebnis (Testset):", title.adj=0.12,
         legend = c(expression(italic(R^2)), 
                    expression(italic("RMSE")),expression(italic("Sensor -")), expression(italic("Kalibrierung")), 
                    expression(italic("Anz. LV")),expression(italic("Anz. WL")),"Anz. Proben",
                    R2_val, RMSE_val, "PLS-R","(Lab)", N.VL_wahl, N.VI_wahl, length(na.omit(y_val))))
  
  name=c("CSB","CSB","BSB","BCSB","CSB","CSB","BCSB","CSB","S","S","X","X")
  indize1=c("","mf","5","","mod","mf,mod","mod","part,mod","S,mod","I,mod","S,mod","I,mod")
  indize2=c("eq",",eq",",eq","eq",",eq",",eq",",eq",",eq",",eq",",eq",",eq",",eq")
  
  title(main = "Korrelationsplot", ylab = substitute(paste(list1[list2]," [mg "* L^-1*"]"), list(list1=name[Name],list2=indize1[Name])), 
        xlab = substitute(paste(list1[list2*list3]," [mg "* L^-1*"]"), list(list1=name[Name],list2=indize1[Name],list3=indize2[Name])))
  box() 
  dev.off()
  
  #Auslesen der Kalbrierung
  wl.name    = c("intercept", (wel.vi*2.5)+wl.select[[2]]-2.5)
  fkt.coef   = as.vector(coef(fkt, ncomp=N.VL_wahl, explvar = TRUE, intercept = TRUE))
  output     = data.frame(wl.name, fkt.coef)
  write.table(output, paste("3_Ausgabe/01_PLS-R/Kalibrierfunktion_", Namensvektor[Name], ".txt", sep=""), sep="\t", row.names = F)
  # #Einlesen der Kalbrierung
  # input = read.table(paste("3_Ausgabe/", Namensvektor[Name], "-Kalibrierfunktion_PLS-R_", best.N.VI, "_", 
  #                            N.VL,".txt", sep=""), sep="\t")
  # wl.pos=NULL
  # wl.name = paste("wl_",input[-1,1],"nm",sep="")
  # for(i in wl.name) wl.pos[i]=which(colnames(X)==i) #suche nach Wellenlängennamen
  # 
  # #Anwenden der Kalbrierung
  # stoffkonzentration = drop(input[1,2] + input[-1,2] %*% t(X[,wl.pos]))
  
  
  # Ladungen an Anteil erklaerter Varianz
  Speicher.Link = paste("2_Abbildungen/01_PLS-R/2 Ladung - PLOT/Ladung_",Namensvektor[Name],".pdf", sep = "")
  pdf(file = Speicher.Link, width = 12.5, height = 9.375)
  ladungen = fkt$loadings[,1]
  
  if(ncol(fkt$loadings)>=4){rr=4}else{rr=ncol(fkt$loadings)} 
  
  for(i in 2:rr){ladungen = rbind(ladungen, fkt$loadings[,i])}
  LV_Anzahl  = 4
  Bar_Breite = as.vector(explvar(fkt)) #Erklarte Varianz
  Bar_Breite_plot = Bar_Breite
  for (i in 1:length(Bar_Breite)) {if(Bar_Breite[i]<2){Bar_Breite_plot[i]=2}; if(Bar_Breite[i]>40){Bar_Breite_plot[i]=20}}
  
  wellenlaengen = as.character(seq(wl.select[[2]],wl.select[[3]],2.5)[wel.vi])
  par(mar=c(12,8,4,4),mgp=c(5,2,0), mfrow=c(1,1), cex.axis=2, cex.lab=2.5, cex.main=3, xpd=FALSE)
  #bp = 
    barplot(as.matrix(ladungen), beside=TRUE, border = "white",
               col=wes_palette("Zissou1", rr, type = "continuous"), width=Bar_Breite_plot, xaxt = "n",
               lwd=4, yaxt = 'n')
  grid(nx = NA, ny = NULL, lwd = 2, col="gray50", lty = 2)
  barplot(as.matrix(ladungen),  beside=TRUE, ylab="", col=wes_palette("Zissou1", rr, type = "continuous"),
          width = Bar_Breite_plot, add = T, xlab = "", las=2, border = "white", xaxt = "n")# names.arg = wellenlaengen)
    
  axis(side = 1, at = (seq(1,((sum(Bar_Breite_plot[1:rr])+5)*length(wellenlaengen)),(((sum(Bar_Breite_plot[1:rr])+5)*length(wellenlaengen))/length(wellenlaengen)))+25), labels = FALSE)
 
  text(x = (seq(1,((sum(Bar_Breite_plot[1:rr])+5)*length(wellenlaengen)),(((sum(Bar_Breite_plot[1:rr])+5)*length(wellenlaengen))/length(wellenlaengen)))+(2.8*length(wellenlaengen))), par("usr")[3]-0.08,
       srt = 45, adj = 1, xpd = TRUE,
       labels = wellenlaengen, cex =2)
  LV_Namen = paste("LV.", seq(1,rr,1)," (", round(x = Bar_Breite, digits = 1), "%)", sep = "")
  mtext(text = "Wellenlänge [nm]", side = 1, line = 6, cex = 2.5)
  test=paste("LV.5"," - LV.",length(Bar_Breite)," (", round(x = sum(Bar_Breite[5:length(Bar_Breite)]),
                                                            digits = 4), "%)", sep = "")
  legend("bottomright", c(LV_Namen[1:rr]), cex=2, fill = c(wes_palette("Zissou1", rr, type = "continuous")),
         inset=0.05, bg = alpha("white", 0.8), ncol=2, title = "LV (inkl. Anteil erklärter Varianz)")
 
  title(main=substitute(paste("Ladungen - " * list1[list2]), list(list1=name[Name],list2=indize1[Name])),
        ylab = "loadings [-]" )
  mtext(text = "Priorisierung ausgewählter Wellenlängen innerhalb Latenter Vektoren (LV)", side = 1, line = 8, cex = 2)
  if(length(Bar_Breite)>5){
    mtext(text = paste("LV.5"," - LV.",length(Bar_Breite)," machen ",
                       round(x = sum(Bar_Breite[5:length(Bar_Breite)]),digits = 4), 
                       "% der erklärten Varianz aus.", sep = ""), side = 1,line = 10, cex = 1.5)
  }else{if(length(Bar_Breite)==5){
    mtext(text = paste("LV.5"," macht ", round(x = sum(Bar_Breite[5:length(Bar_Breite)]),
                                               digits = 4), "% der erklärten Varianz aus.", sep = ""), side = 1,line = 10, cex = 1.5)}else{}}
  box()
  dev.off()
  
  #Auslesen der Ladung
  wl.name    = c(wel.vi*2.5)+wl.select[[2]]-2.5
  fkt.load   = loadings(object = fkt, ncomp=N.VL_wahl)
  colnames(fkt.load) = paste("LV_",seq(1,length(Bar_Breite),1))
  output     = data.frame(wl.name, fkt.load[,1:length(Bar_Breite)])
  write.table(output, paste("3_Ausgabe/01_PLS-R/Ladung_", Namensvektor[Name],".txt", sep=""), sep="\t", row.names = F)
  
  
  
  ###############################################################
  # Bootstrap-Verteilung von Regressionskoeffizienten
  #     Regressionskoeffizienten aus allen Bootstrap-Proben fuer die Extinktionsdaten unter 
  #     Verwendung von PLS-R mit dem Optimum an latenten Vektoren (LV) und Wellenlaengen (WL).
  nBoot = 1000
  nsample = nrow(XE.tri)
  boot.indices = matrix(sample(1:nsample, nsample * nBoot, replace = TRUE), ncol = nBoot)
  plsr = plsr(y_kal ~ as.matrix(XE.tri), ncomp = N.VL_wahl)
  coefs = matrix(0, ncol(XE.tri), nBoot)
  fit_val = matrix(0, nrow(XE.tri), nBoot)
  for (i in 1:nBoot) {
    bootplsr = plsr(y_kal ~ as.matrix(XE.tri), ncomp = N.VL_wahl, subset = boot.indices[,i])
    fit_val[,i] = c(bootplsr$fitted.values[,,N.VL_wahl])
    coefs[,i] = c(coef(bootplsr))
  }
  Daten = t(coefs[1:nrow(coefs),])
  colnames(Daten)=wellenlaengen
  Speicher.Link = paste("2_Abbildungen/01_PLS-R/3 Konfidenz - PLOT/Konfidenz_Bootstrap_",Namensvektor[Name], ".pdf", sep = "")
  pdf(file = Speicher.Link, width = 12.5, height = 9.375)
  par(mar=c(11,8,4,20),mgp=c(5,2,0), mfrow=c(1,1), cex.axis=2.5, cex.lab=2.5, cex.main=3, xpd=FALSE)
  boxplot(Daten, xlab="", pch=20 , cex=0.3 , ylab="",
          xaxt="n", xaxs = FALSE, lwd=2.5, cex.axis=2.5, cex.lab=2.5, cex.main=3)
  grid(nx = NA, ny = NULL, lwd = 2, col="gray50", lty = 2); abline(0,0, lwd = 2, col="gray50"); par(new=TRUE)
  boxplot(Daten, col = wes_palette("Zissou1", 1, type = "continuous"), pch=20 , cex = 0.3, xaxt = "n",
          xaxs = FALSE, lwd = 2.5, cex.axis = 2.5, cex.lab = 2.5)
  abstand=(max(Daten)-min(Daten))*0.05
  text(x = c(1:length(wellenlaengen)), par("usr")[3]-abstand,
       srt = 45, adj = 1, xpd = TRUE,
       labels = wellenlaengen, cex = 2.5)
  axis(side = 1, at = c(1:length(wellenlaengen)), labels = FALSE)
  points(plsr$coefficients[,,N.VL_wahl], pch = 16, col = col_vec[2], cex = 1.5)
  color = "white"
  color_transparent = adjustcolor(color, alpha.f = 0.2)
  par(xpd=TRUE)
  legend("topright",pch = c(22,NA,NA,16,16), col = c( "black", "black","black","black",col_vec[2]),
         legend = c("25-75%","9-91%","Mittelwert","Ausreißer","Beste Kombination"), horiz = FALSE,
         bg = alpha("white", 0.9), inset = c(-0.55,0), cex=1.75, lty = c(NA,1,1,NA,NA),lwd = c(2,2,7,2,2),
         pt.cex = c(6,NA,NA,1,2), pt.bg = c(wes_palette("Zissou1", 1, type = "continuous"),NA,NA,NA,NA ),
         bty = "n")
  title(main = substitute(paste("Konfidenzintervalle - " * list1[list2]), list(list1=name[Name],list2=indize1[Name])),
        ylab = "Regressionskoeffizient [-]")
  mtext(text = "Wellenlänge [nm]", side = 1, line = 7, cex = 2.5)
  mtext(text = expression(italic("Bootstrap-Verfahren")*" - Unsicherheit der Koeffizienten einzelner WL"), side = 1, line = 9.5, cex = 2)
  dev.off()
  #Auswertung:
  #Hohe Unsicherheit im Vis Spektralbereich        -> Einflussnahme der Kolloide und TSS
  #Hohe Unsicherheit im UV Spektralbereich (200nm) -> Spaltbreite des Messfenster falsch.
  
  
  ###############################################################
  #---Guete des Modells ----
  ###############################################################
  n = Anz_sample             #Anzahl an Beobachtungen
  p = as.numeric(N.VL_wahl)  #Anzahl an Einflußvariablen
  d_f = n-p-1                #Freiheitsgrade
  #Guete der Regressionsfunktion als Maß ihrer Anpassung an die empirischen Daten 
  #Bestimmtheitsmaß
  #Korrelation ist die standardisierte Kovarianz
  R2 = round(cor(x = y_val_fit, y = y_val, method = "pearson", use = "pairwise.complete.obs")^2, digits = 3)
  #   erklärte Variation dividiert durch die Gesamtvariation
  #   Streuung der Modellvohersagen (Schätzwerte) um das Gesamtmittel dividiert durch die Streuung der 
  #   Messwerte um das Gesamtmittel
  #Korrigiertes Bestimmtheitsmaß
  R2_adj = 1-(1-R2)*(n-1)/(n-p-1)
  
  #Root Mean Squares Error
  #RMSE = round(Metrics::rmse(actual = y_val, predicted = y_val_fit), digits = 3)
  #   rmse = sqrt(mse)                   #Mittleres Abweichungsquadrat
  #   mse  = mean(se(actual, predicted)) #mittlerer Standardfehler 
  #   se   = (actual - predicted)^2      #Standardfehler oder Streuungsmass
  
  #F-Wert
  #   Die F-Statistik ist ein Indikator eine Beziehung zwischen den Prädiktorvariablen 
  #   und den Antwortvariablen zu pruefen. Je weiter die F-Statistik von 1 entfernt ist, desto besser ist sie. 
  #   Wie viel größer die F-Statistik sein muss, hängt von der Anzahl der Datenpunkte als auch 
  #   von der Anzahl der Prädiktoren ab. Wenn die Anzahl der Datenpunkte groß ist, reicht im Allgemeinen 
  #   bereits eine F-Statistik aus, die nur wenig größer als 1 ist, um die Nullhypothese abzulehnen 
  #   (H0: Es gibt keine Beziehung zwischen Konzentration und Extinktion). 
  F.emp = (R2/(1-R2))*((n-p-1)/p)
  #   Die Nullhypothese des F-Tests besagt, dass alle Koeffizienten gleich 0 sind. Hingegen ist die Alternative, 
  #   dass mindestens ein Koeffizient ungleich 0 ist – es also mindestens eine Kovariate im Modell gibt, die 
  #   signifikanten Einfluss auf die abhängige Variable ausuebt. Die Nullhypothese wird abgelehnt, wenn der p-Wert 
  #   kleiner als ein gewähltes Signifikanzniveau ist:
  p.wert = pf(F.emp, p, d_f, lower.tail = FALSE)
  alpha=0.05 #Signifikanzniveau 
  if(p.wert<alpha){print("Nullhypothese wird abgelehnt. Modell signifikant. Es existiert min. eine Kovariate im Modell.")}
  
  #t-Wert
  # t-Wert gibt Auskunft ueber die Signifikanz einzelner Regressionskoeffizienten (WL).
  #https://rdrr.io/cran/pls/man/jack.test.html
  # Jackknife-Varianzschaetzungen sind verzerrt. Verteilung der Regressionskoeffizientenschätzungen
  # und Jackknife-Varianzschätzungen ist unbekannt. 
  # Folglich ist die Verteilung (und insbesondere die Freiheitsgrade) der 
  # resultierenden t-Statistik unbekannt. Der vorliegende Code nimmt einfach eine 
  # t-Verteilung mit m - 1 Freiheitsgraden an, wobei m die Anzahl der Kreuzvalidierungssegmente ist.
  # resultierende p-Werte nur Indikator fuer (Nicht-) Signifikanz.
  # Annahme: Regressoren, die innerhlab der forward Methode ausgwählt werden, sind signifikant fuer das Modellergebnis
  #  Aussage darueber, welche WL besonders hervorzuheben ist
  jt = jack.test(object = fkt, ncomp = N.VL_wahl, use.mean = F)
  
  GueteMat = matrix(NA, ncol = 13, nrow = N.VI_wahl)
  GueteMat[1,7:13] = c(p.wert,R2_kal,R2_adj_kal,RMSE_kal,R2_val,R2_adj_val,RMSE_val)
  GueteMat[,2:5]   = cbind(jt$coefficients, jt$sd, jt$tvalues, jt$pvalues)
  GueteMat[1,6]    = N.VL
  #Auslesen der Modellguete
  GueteMat[,1]    = c(wel.vi*2.5)+wl.select[[2]]-2.5
  colnames(GueteMat) = c("WL", "Koeffizient", "Std. Error", "t value", "Pr(>|t|)", "LV","p-Wert von F-Test", 
                         "R2 kal","R2_adj kal","RMSE kal","R2 val","R2_adj val","RMSE val")
  output     = data.frame(GueteMat)
  write.table(output, paste("3_Ausgabe/01_PLS-R/Modellguete_", Namensvektor[Name], "_n",length(y_val),".txt", sep=""), sep="\t", row.names = F)
  
  
  ###############################################################  
  #----Modellprämissen ----
  ###############################################################
  #Linearität
  #:= Lambert-Beer beschreibt einen linearen Zusammenhang zwischen Ext. und Stoffkonz.
  #   Verteilung der Residuen ueber die berchneten Werte
  #   TEST: Harvey-Collier-Test: Mittelwert der rekursiven Residuen beträgt NULL
  #   TEST: Rainbow-Test
  Speicher.Link = paste("2_Abbildungen/01_PLS-R/4 Modellpraemissen/Linearitaet_",Namensvektor[Name], ".pdf", sep = "")
  pdf(file = Speicher.Link, width = 12.5, height = 9.375)
  residuen=resid(object = fkt)
  par(mar=c(7,9,4,5),mgp=c(5,2,0), mfrow=c(1,1), cex.axis=2.5, cex.lab=2.5, cex.main=3, xpd=FALSE)
  plot(x = y_kal_fit, y = residuen[,,N.VL_wahl], xlim = range(0,max(y_kal_fit, y_kal)), 
       ylab = "", xlab = "")
  reg=lm(as.vector(residuen[,,N.VL_wahl])~as.vector(y_kal_fit))
   grid(nx = NULL, ny = NULL, lwd = 2, col="gray50", lty = 2) 
  abline(h = 0, lwd = 2, col="gray50", lty = 1)
  points(x = y_kal_fit, y = residuen[,,N.VL_wahl], pch=16, cex=2)
  lines(as.vector(y_kal_fit),reg$fitted.values, type = "l",lty = 2, lwd=5, col=col_vec[2])
  title(main = paste("Residuen vs. Modellvorhersage", sep = ""), xlab = expression("Sensor [mg "* L^-1*"]" ), 
        ylab = expression(Residuen *"[mg "* L^-1*"]" ))
  dev.off()
  #Beobachtungen: Mit zunehmender Stoffkonzentration ist eine steigende Varianz der Residuen zu erkennen, die gleichmäßig
  #               in positiver und negativer Richtung verteilt ist. Der Korrelationsplot bestätigt die Linearität.
  
  #Multikollinearität
  #:= Multikollinearität zwischen den Prädiktorvariablen. 
  #   Korrelationstabelle
  #   Varianve Inflation Factor (VIF) fuer jede unabhängige Variable; Ergebnis: je jöher der Wert umso höher die multiple Korrelation 
  Speicher.Link = paste("2_Abbildungen/01_PLS-R/4 Modellpraemissen/Multikollinearitaet_Korrelationsplot_",Namensvektor[Name], ".pdf", sep = "")
  pdf(file = Speicher.Link, width = 12.5, height = 9.375)
  par(mar=c(11,8,4,8),mgp=c(5,2,0), mfrow=c(1,1), cex.axis=1.5, cex.lab=2, cex.main=3, xpd=FALSE)
  library('plot.matrix')
  x=cor(X[,wel.vi])
  colnames(x) = (wel.vi*2.5)+wl.select[[2]]-2.5
  rownames(x) = (wel.vi*2.5)+wl.select[[2]]-2.5
  class(x)
  plot(x, breaks=c(0.5,0.8,0.85,0.9,0.93,0.97, 0.99, 0.999,1), xlab="[nm]", ylab="[nm]", xaxt="n",#border=NA,
       col =  c("grey90","grey80","grey70","grey60",
                "grey50","grey30","grey25","grey20"), main = "Korrelationsplot - Wellenlängen")
  if(length(wel.vi)>9){
    ttt=paste(as.character((wel.vi[1:9]*2.5)+wl.select[[2]]-2.5), collapse=", ")
    mtext(text = paste("Signifikante Wellenlängen:",ttt, collapse=", "), side = 1, line = 7, cex = 1.75)
    tt=paste(as.character((wel.vi[10:length(wel.vi)]*2.5)+wl.select[[2]]-2.5), collapse=", ")
    mtext(text = tt, side = 1, line = 9, cex = 1.75)
  }else{
    ttt=paste(as.character((wel.vi[1:length(wel.vi)]*2.5)+wl.select[[2]]-2.5), collapse=", ")
    mtext(text = paste("Signifikante Wellenlängen:",ttt, collapse=", "), side = 1, line = 7, cex = 2)
  }
  box()
  dev.off()
  #Normalverteilung der Störgrößen 
  #:= Normalverteilung der gesuchten Größe; Normalverteilung wichtig fuer F- und t-test
  #   TEST: Shapiro-Wilk-Test und Kolmogorow-Smirnow-Test
  #   Darstellung ueber ein QQ-Plot: standardisierte Residuen, die Laut Normalverteilung zu erwarten wären geplotet. Histogramm der Residuen mit einer Dichtefunktion
  #   Ergebnisse empirischer Tests (Shapiro-Wilk-Test und Kolmogorow-Smirnow-Test) bei 
  #    größeren Datensätzen ergeben häufig eine Nicht-Normalverteilung. Grafische Überpruefung mit ergeben häufig gegenteiliges.
  #   Kolmogorow-Smirnow-Test (KS-Test) 
  #   Frage: Weisen zwei Datensätze die selbe Verteilung auf (z.B. Normalverteilung)?
  #   Statistischer Test auf Uebereinstimmung zweier Wahrscheinlichkeitsverteilungen.
  #   Nachteil: es wird nicht deutlich, welche Verteilung vorliegt.
  #   je kleiner P-Wert umso unterschiedlicher Verteilungen
  ks = ks.test(x = y_val, y = y_val_fit)
  #   Shapiro-Wilk-Test (aehnlicher Test)i.d.R. eine höhere statistische Power; ist dem ks.test vorzuziehen 
  nummer = 5
  s.t = shapiro.test(x = X_kal[,nummer])
  #   Nullhypothese (H0) besagt bei diesem Test, dass eine Normalverteilung vorliegt.
  #   Alternativhypothese (H1): keine Normalverteilung; Signifikanz (p < 0.05): keine Normalverteilung
  print(s.t)
  
  Speicher.Link = paste("2_Abbildungen/01_PLS-R/4 Modellpraemissen/Normalverteilung_QQ-Plot_",Namensvektor[Name], "_PLS-R.pdf", sep = "")
  pdf(file = Speicher.Link, width = 12.5, height = 9.375)
  par(mar=c(7,9,4,5),mgp=c(5,2,0), mfrow=c(1,1), cex.axis=2.5, cex.lab=2.5, cex.main=3, xpd=FALSE)
  y_val_stand = robustHD::standardize(x = residuen[,,N.VL_wahl], centerFun = mean,  scaleFun = sd)  
  qqnorm(y = y_val_stand, pch = 16, xlab = "", ylab = "", cex = 2, main = "")
  grid(nx = NULL, ny = NULL, lwd = 2, col="gray50", lty = 2); abline(h = 0, lwd = 2); abline(v = 0, lwd = 2)
  qqline(y_val_stand, col = "steelblue", lwd = 2)
  title(xlab = "Theoretisches Quantil", ylab = expression("Standardisierte Residuen [mg "* L^-1*"]" ), main = "Q-Q Plot - Normalverteilung")
  dev.off()
  
  #Histogramm der Residuen
  Speicher.Link = paste("2_Abbildungen/01_PLS-R/4 Modellpraemissen/Normalverteilung_Histo_",Namensvektor[Name], ".pdf", sep = "")
  pdf(file = Speicher.Link, width = 12.5, height = 9.375)
  par(mar=c(7,8,4,5),mgp=c(5,2,0), mfrow=c(1,1), cex.axis=2.5, cex.lab=2.5, cex.main=3, xpd=FALSE)
  hist(residuen[,,N.VL_wahl], main = "Histogramm der Residuen", freq = F,
       xlab = expression("Residuen [mg "* L^-1*"]" ), ylab = "Dichte [-]", breaks = 10)
  rug(jitter(residuen[,,N.VL_wahl]), col="brown")
  curve(dnorm(x, mean=mean(residuen[,,N.VL_wahl]), sd=sd(residuen[,,N.VL_wahl])), add=TRUE, col=col_vec[1], lwd=2)
  lines(density(residuen[,,N.VL_wahl])$x, density(residuen[,,N.VL_wahl])$y, col=col_vec[2], lwd=2, lty=2)
  legend("topright", legend = c( "Normalverteilung", "Dichtefunktion der Residuen"),
         lty=1:2, col=col_vec, bg = alpha("white", 0.9), inset = 0.05, cex=1.75)
  box()
  dev.off()
  
  vec_normal = c("Normalverteilung", "Kolmogorow-Smirnow",if(ks$p.value<0.05){c("erfuellt","Normalverteilung")}else{c("nicht erfuellt","nicht normalverteilt")})
  
  #Homoskedastizität
  #:= Varianz der Residuen ist konstant. Heteroskedastizität liegt vor wenn sie nicht konstant sind.
  #Goldfeld/Quandt-Test: statistischer Test auf Heteroskedastizität 
  #Bartlett-Test: werden zwei verschiedene statistische Tests bezeichnet: der Bartlett-Test auf Gleichheit 
  #               der Varianzen in Stichproben und der Bartlett-Test auf Sphärizität zur Durchfuehrung einer 
  #               Faktorenanalyse.
  #               Frage: Ist eine Datenreduktion z.B. in Form einer PLS-R moeglich?
  #               sicherstellen, dass die Korrelationsmatrix der Variablen in unserem Datensatz erheblich von der 
  #               Identitätsmatrix (linear unabhängig) abweicht, sodass wir wissen, dass eine Datenreduktionstechnik 
  #               geeignet ist. Wenn der p-Wert aus Bartletts Sphärizitätstest niedriger 
  #               ist als das von uns gewählte Signifikanzniveau (uebliche Auswahlmöglichkeiten sind 0,10, 0,05 und 0,01), 
  #               ist unser Datensatz fuer eine Datenreduktionstechnik geeignet.
  #               Voraussetzungen:
  #               - Variablen sind in der Grundgesamtheit normalverteilt
  #               - Pruefgröße folgt näherungsweise einer Chi²-Verteilung
  bartlett.test = REdaS::bart_spher(x = X, use = c("pairwise.complete.obs"))
  if(bartlett.test$p.value<alpha){print("Nullhypothese wird abgelehnt. Regressoren fuer Datenreduktion geeignet.")}
  #Levene-Test (ähnlicher Test); bessere Wahl fuer Daten, die nicht normal verteilt sind
  #               Frage: Gleichheit der Varianzen (Homoskedastizität)?
  levene_test = matrixTests:: col_levene(x = X, g = Y)
  any(levene_test$pvalue>0.05)==TRUE
  #Wellenlängenbereich mit p-value > 0.05 werden innerhlab der PLS als wichtig erkannt. Reduktionsfähige Regressoren werden durch nur eine WL vertreten
  #Grafische Pruefung der Homoskedastizität
  #Darstellung ueber ein Scale-Location Diagramm: standardisierte Residuen mit berechneten Werten plotet. 
  #Ergebnis: Horizontale ansonsten liegt Heteroskedastizität vor
  Speicher.Link = paste("2_Abbildungen/01_PLS-R/4 Modellpraemissen/Homosked_",Namensvektor[Name], "_PLS-R.pdf", sep = "")
  pdf(file = Speicher.Link, width = 12.5, height = 9.375)
  par(mar=c(7,9,4,5),mgp=c(5,2,0), mfrow=c(1,1), cex.axis=2.5, cex.lab=2.5, cex.main=3, xpd=FALSE)
  
  y_val_stand = robustHD::standardize(x = residuen[,,N.VL_wahl], centerFun = mean,  scaleFun = sd)  
  
  plot(x = y_kal_fit, y = sqrt(x = y_val_stand ), xlim = range(0,max(y_kal_fit)), ylab = "", xlab = "")
  grid(nx = NULL, ny = NULL, lwd = 2, col="gray50", lty = 2) 
  y = sqrt(x = y_val_stand);Y=na.omit(cbind(y, y_kal_fit))
  reg=lm(Y[,1] ~ Y[,2])
  lines(Y[,2], reg$fitted.values, col=col_vec[1], lwd = 2)
  points(x = y_kal_fit, y = sqrt(y_val_stand), pch=16, cex=2)
  title(main = paste("Scale-Location-Plot", sep = ""), xlab = expression("Sensor [mg "* L^-1*"]" ), 
        ylab = expression(sqrt(Standardisierte_Residuen) *"[mg "* L^-1*"]" ))
  dev.off()
  
  vec_homo = c("Homoskedastizität", "Barlett-Test",if(bartlett.test$p.value<0.05){c("erfuellt","Homoskedastizität")}else{c("nicht erfuellt","Heteroskedastizität")})
  
  
  
  
  #Autokorrelation
  #:= Unabhängigkeit der Residuen; bei Zeitreihen mit dicht aufeinander folgenden Datenpunkten ist die Wahrscheinlichkeit 
  #   einer Autokorrelation hoch. TEST: Durbin-Watson-Test  
  dw = dwtest(fkt, alternative="two.sided") #(Hothorn et al.2014)
  #Die Durbin-Watson-Statistik kann Werte zwischen 0 und 4 annehmen. Bei einem Wert von 2 existiert keine Autokorrelation zwischen den Residuen. Da Statistiken selten exakte Werte annehmen werden, können wir auch bei Werten nahe 2 davon ausgehen, dass keine Autokorrelation vorliegt.
  #The null hypothesis (H0) is that there is no correlation among residuals
  #Die alternative Hypothese (Ha) besagt, dass Autokorrelation der Residuen vorliegt.
  #Wenn einer der p-Werte <0,05 (oder ausgewähltes Alpha) ist, wissen wir, dass die entsprechende alternative Hypothese wahr ist
  if(dw$p.value>0.05){print("Alternative Hypothese wird abgelehnt: Keine Autokorrelation der Residuen!")}
  
  vec_auto = c("Autokorrelation", "Durbin-Watson-Test",if(dw$p.value>0.05){c("erfuellt","keine Autokorrelation")}else{c("nicht erfuellt","Autokorrelation")})
  
  output = rbind(vec_normal, vec_auto, vec_homo)
  colnames(output) = c("Guetekriterium", "Test","Bedingung", "Ergebnis") 
  
  output     = data.frame(output)
  write.table(output, paste("3_Ausgabe/01_PLS-R/Modellpraemisse_", Namensvektor[Name],"_n",length(y_val), ".txt", sep=""), sep="\t", row.names = F)
  
  
  summary_mat[Name,] = c(R2_val,RMSE_val)
  #Name=Name+1

#  }
#write.table(x = summary_mat, paste("3_Ausgabe/01_PLS-R/Modellguete.txt", sep=""), sep="\t", row.names = T, dec = ",")


  
  
  

  ###############################################################_
  #----Metadaten-PLOT
  # plot.meta(X = "CSBmf", Y = "CODfeq", Daten = DAT.select, Metadatenkategorie = "Seriennummer", Zeitvektor = c(1,2,3,6,12,24,48,72), 
  #                      Speichern = T, Speichername = "TEST")
  #Beschreibung: Korrelationsplot, in dem eine Unterscheidung zwischen Metainformationen vorgenommen werden kann 
  #X = "CSBmf"
  #Y = "CODfeq"
  #Optionen: "CSBmf","CSB","BSB5","BCOD","NO3.N","TSSeq","Status_TSSeq","NO3.Neq","Status_NO3.Neq","CODeq",
  #          "Status_CODeq","CODfeq","Status_CODfeq","SAC254","Status_SAC254","SAC436","Status_SAC436","X254.436",
  #          "Status_254.436","analogIN","Status_analogIN","wl_200nm" - "wl_737.5nm","COD","BCOD.1","RBCOD","SBCOD",
  #          "COD.1","CODf","CODpart","Ss","Si","Xs","Xh","Xi" 
  #Daten = DAT.select
  #Metadatenkategorie = "Bearbeiter"   #c("Zeitvektor","Versuchsnummer","Bearbeiter","Seriennummer","Messstandort")
  #Speichern = T
  #Speichername = "TEST"
  #Zeitvektor = c(1,2,3,6,12,24,48,72)
 
  
  
  
  # Speicher.Link = "2_Abbildungen/01_PLS-R/4 Modellpraemissen/Korrelationsplot_komplett.pdf"
  # pdf(file = Speicher.Link, width = 12.5, height = 9.375)
  # par(mar=c(8,8,4,10),mgp=c(5,2,0), mfrow=c(1,1), cex.axis=2, cex.lab=2.5, cex.main=3, xpd=FALSE)
  # 
  # x=cor(X)
  # colnames(x) = seq(220,732.5,2.5)
  # rownames(x) = seq(220,732.5,2.5)
  # class(x)
  # plot(x, breaks=c(0.5,0.8,0.93,0.97,0.99,0.999, 0.9999, 0.99999,1), xlab="[nm]", ylab="[nm]", xaxt="n",border=NA,
  #      col =  c("grey90","grey80","grey70","grey60",
  #               "grey50","grey30","grey25","grey20"), main = "")
  # box()
  # dev.off()
  
  
  
  
  ##############################
  #  HIER GEHT ES WEITER