# Kalibrierfunktionen mit Partcial Least Squares Regression (PLS-R) 
# Beschreibung: PLS-R nach dem Ansatz von Torres 2008. Messreihen aus Respirationsversuchen 
#                  
# Ergebnis:         - Kalibrierfunktionen als *.txt 
#                   - Faktorladungen als *.txt 
#                   - Modellpraemissen, Regressionsbewertung als *.txt
#                   - Plots: o Korrelation zw. PLS-R Ergebnis (IST) und Referenz (SOLL) 
#                            o Faktorladungen 
#                            o Bootstarpping fuer Konfidenzintervall

###############################################################_
#---BIBLIOTHEK - Funktionen ----
source(file = "../FUNKTIONEN.R")
# 
#----Lokale Funktionen ----
NA_in_0     = function(x)      {POS_NA = which(is.na(x)==TRUE); x[POS_NA] = 0; return(x)} #Umwandlung von NA in 0
NA_all      = function(x,Spalte)      {if(is.vector(x)){NA_TRUE=sum(is.na(x));length(x)==NA_TRUE}else{NA_TRUE=sum(is.na(x[,Spalte])); length(x[,Spalte])==NA_TRUE}}
euc.dist    = function(x1, x2) {sqrt(sum((x1 - x2) ^ 2))} #Berechnung der euklidischen Distanz
is.integer0 = function(x)      {is.integer(x) && length(x) == 0L} #Pruefung ob Variable ein integer ist TRUE/FALSE
delete.na   = function(DF, n=0) {DF[rowSums(is.na(DF)) <= n,]} #Loeschen von Zeilen mit definierter Anzahl an NA?s

###############################################################_
#----BIBLIOTHEK - package  ----
library(ISI.Toolbox);library(zoo);library(dygraphs);library(MASS);library(XLConnect);library(matrixStats)
library(ggplot2);library(scales);library(Metrics);library(wesanderson);library(gridExtra);library(grid);
library(pls);library(cluster);library(arm);library(lmtest);library(car);library(glmnet); library(lars)

###############################################################_
#----Datengrundlage wird eingelesen ----
Versuchstage = c("2013-03-11","2013-04-02","2013-04-09","2014-04-08","2014-04-28","2014-05-05","2019-02-21","2018-02-13",
                 "2018-03-05","2019-02-25","2019-02-26","2019-03-05","2019-04-08","2019-04-10","2020-03-05")
DAT    = read.table(file = "1_Daten/Gesamt.txt", sep="\t", header = TRUE, dec = ".")
Datime = strptime(DAT$Datime, format="%Y-%m-%d %H:%M:%S", tz="UTC")
DAT    = data.frame(Datime, DAT[,-1])

###############################################################_
#----Selektion aus Datengrundlage ----
Namensvektor = c("CSB","CSBmf","BSB5","BCSB","CSB_mod",
                 "CSBmf_mod","BCSB_mod","CSBpart_mod","Ss_mod","Si_mod",
                 "Xs_mod","Xi_mod")
Name = 1
 summary_mat = matrix(data = NA, nrow = length(Namensvektor), ncol = 8)
 colnames(summary_mat) = c("R2_kal", "R2_kal_adj","RMSE_kal","R2_val", "R2_val_adj","RMSE_val","t/t_o", "Anz.WL")
 rownames(summary_mat) = Namensvektor
summary_mat=read.table(file = "3_Ausgabe/02_Lasso-R/Modellguete.txt", header = TRUE, sep = "\t", dec = ",")


#for (Name in 1:4){#length(Namensvektor)) {
  
  DPproIntervall=50
  # DAT.select = dat.select(Daten = DAT,Selektparameter = c(Namensvektor[Name],"wl_435nm"), Anz.pro.Zeitabschnitt = DPproIntervall,
  #                         Zeitvektor = c(1,2,3,6,12,24,48,72))
  Ordner = "3_Ausgabe/02_Lasso-R/"
  File1   = paste("DF_select_",DPproIntervall,"_",Namensvektor[Name],".txt",sep = "")
  DAT.select = read.table(file = paste(Ordner, File1, sep = ""), sep="\t", header = TRUE)
  
  dim(DAT.select)
  #Auswahl des Regressanden
  #Auswahl und ggf. Eingrenzung der Regressoren
  #   Wellenlaengen aus Datengrundlage obere_wl = 200, untere_wl = 732.5
  wl.select = wl_auswahl(Daten = DAT.select, Datenauswahl = "wl-bereich", obere_wl = 220, untere_wl = 732.5)
  x        = wl.select[[1]]
  
  #Loeschen von Werten, die Null betragen 
  y.pos      = which(colnames(DAT.select)==Namensvektor[Name])
  y          = DAT.select[,y.pos]   #Regressand: Parameter
  y.null.pos = which((y > 0) & (!is.na(y)))
  Y          = y[y.null.pos]
  X          = x[y.null.pos,]
  
  ##### Welche Werte werden fuer die Kalibrierung und Valdierung verwendet?
  File   = paste("DF_KalVal_",DPproIntervall,"_",Namensvektor[Name],".txt",sep = "")
  # if(is.integer0(which(dir(Ordner) == File))){
  #   kal_row = sample(x = seq(1, nrow(X),1), size = round(x = (nrow(X)*0.6), digits = 0), replace = FALSE)
  #   row     = seq(1, nrow(X),1)
  #   val_row = setdiff(row, kal_row)
  #   datenaufteilung = matrix(data = NA, nrow = length(kal_row), ncol = 2)
  #   datenaufteilung[,1]=kal_row; datenaufteilung[1:length(val_row),2]=val_row
  #   colnames(datenaufteilung) = c("Kalibrierung", "Valdierung")
  #   write.table(datenaufteilung, file = paste(Ordner, File), sep="\t", row.names = TRUE)
  # }else{
    kal_val = read.table(file = paste(Ordner, File, sep = ""), sep="\t", header = TRUE)
    kal_row = kal_val$Kalibrierung
    val_row = kal_val$Valdierung
  # }
  
  #Aufteilung in Kalibrierungs un Validierungsdatensatz
   X_kal   = as.matrix(X[kal_row,]); X_val = as.matrix(X[val_row,]); y_kal = y[kal_row]; y_val = y[val_row]
  
  
  ################################################################
  #Kalibrierung (train) des Lasso-Regressionsmodells ist genau das gleiche wie das der Ridge-Regression. 
  lasso.fkt  = lars(X_kal, y_kal, type = "lasso", trace=TRUE, use.Gram=TRUE, intercept = FALSE)
    #s = beta/beta_o oder auch t/t_0
    #          t_0: Summe aller Ausgangskoeffizienten
    #          t oder beta: Grenzwert für die Summe der Beträge der Koeffizienten, die nicht überschritten wird 
  # plot(x = lasso.fkt,breaks = TRUE, plottype = "coefficient", xvar = "norm", xlim=range(0,0.02),
  #      ylim=range(-10000,10000), ylab = "NULL")
  s.hat      = 0.01
  y_kal_fit  = as.vector(predict(lasso.fkt, X_kal, s.hat, type="fit", mode="fraction")$fit)

  co         = predict(lasso.fkt, type="coefficients", mode="fraction", s = s.hat)
  wel.lars   = names(co$coefficients[which(co$coefficients!=0)])
  vec.pos    = which(colnames(X_kal) %in% wel.lars)
  out        = data.frame(vec.pos,co$coefficients[which(co$coefficients!=0)])
  names(out) = c("Pos", "Koeff")
  meth       = paste("Lasso",Namensvektor[Name],s.hat, sep="_")
  
  y_val_fit  = predict(lasso.fkt, s = s.hat, mode="fraction", newx = X_val) #Validierungsdatensatz

  
  ###############################################################
  #----Lasso-R Auswertung und graphische Darstellellung ----
  ###############################################################
  # Korrelationsplot
  #Kalibrierung
  N.VL = length(vec.pos)
  RMSE_kal   = round(Metrics::rmse(actual = y_kal, predicted = y_kal_fit), digits = 3)
  R2_kal     = round(cor(x = y_kal_fit, y = y_kal, use = "pairwise.complete.obs", 
                         method = "pearson")^2, digits = 3)
  Anz_sample = dim(X_kal)[1]
  R2_adj_kal = round(1-(1-R2_kal)*(Anz_sample-1)/(Anz_sample-N.VL-1), digits = 3)
  #Validierung
  DF_Val=na.omit(cbind(y_val_fit$fit,y_val))
  R2_val         = round(cor(x = DF_Val[,1], y = DF_Val[,2], use = "pairwise.complete.obs", 
                             method = "pearson")^2, digits = 3)
  RMSE_val       = round(Metrics::rmse(actual = DF_Val[,1], predicted = DF_Val[,2]), digits = 3)
  Anz_sample = dim(X_val)[1]
  R2_adj_val     = round(1-(1-R2_val)*(Anz_sample-1)/(Anz_sample-N.VL-1), digits = 3)
  
  print(paste("Ergebnisse der Kalibrierung: "," R2=", R2_kal, " R2_adj=",R2_adj_kal," RMSE=", RMSE_kal))
  print(paste("Ergebnisse der Valdierung: "," R2=", R2_val, " R2_adj=",R2_adj_val," RMSE=", RMSE_val))
  
  Speicher.Link = paste("2_Abbildungen/02_Lasso/1 Korrelation - PLOT/Kor_Lasso-R-Kal_",Namensvektor[Name],".pdf", sep = "")
  pdf(file = Speicher.Link, width = 12.5, height = 9.375)
  col_vec = wes_palette("Zissou1", 2, type = "continuous")
  par(mar=c(6,8,4,4),mgp=c(5,2,0), mfrow=c(1,1), cex.axis=2.5, cex.lab=2.5, cex.main=3, xpd=FALSE)
  x = as.vector(y_kal_fit); y = y_kal
  plot(x = x, y = y, xlim = range(0, max(y_kal_fit,y_val_fit$fit,y_kal,y_val,na.rm = T)), xlab ="", ylab="" ,
       ylim = range(0, max(y_kal_fit,y_val_fit$fit,y_kal,y_val,na.rm = T)), pch = 16, lwd = 6)
  axis(1); axis(2)
  grid(nx = NULL, ny = NULL, lwd = 2, col="gray50", lty = 2) 
  abline(0,1, col="gray50", lty=1, lwd =2)
  points(x = x, y = y, pch=15, col = col_vec[1], lwd=2, cex = 3)
  points(x = y_val_fit$fit, y = y_val, pch=16, col = col_vec[2], lwd=2, cex = 3)
  legend("topleft", legend = c("Trainingsset", "Testset"), col = col_vec, 
         inset = 0.025, bg = scales::alpha("white", 0.8), cex=2, pch = c(15, 16), pt.cex = 3)
  legend('bottomright', ncol = 2, bg = scales::alpha("white", 0.8), inset = 0.025, cex = 1.75, 
         title = "Ergebnis (Testset):", title.adj=0.12,
         legend = c(expression(italic(R^2)), 
                    expression(italic("RMSE")),expression(italic("Sensor -")), expression(italic("Kalibrierung")), 
                    expression(italic("t/t_0")),expression(italic("Anz. WL")),"Stichproben",
                    R2_val, RMSE_val, "Lasso-R","(Lab)", s.hat, N.VL, length(na.omit(y_val))))
  
  name=c("CSB","CSB","BSB","BCSB","CSB","CSB","BCSB","CSB","S","S","X","X")
  indize1=c("","mf","5","","mod","mf,mod","mod","part,mod","S,mod","I,mod","S,mod","I,mod")
  indize2=c("eq",",eq",",eq","eq",",eq",",eq",",eq",",eq",",eq",",eq",",eq",",eq")
  
  title(main = "Korrelationsplot", ylab = substitute(paste(list1[list2]," [mg "* L^-1*"]"), list(list1=name[Name],list2=indize1[Name])), 
        xlab = substitute(paste(list1[list2*list3]," [mg "* L^-1*"]"), list(list1=name[Name],list2=indize1[Name],list3=indize2[Name])))
  box() 
  dev.off()

  

  plot(lasso.fkt, xvar="step", xlim = range(0, 6), ylim = range(-3000, 3000),lwd = 2)
 
  
  
  
    
  #Sicherung der Ergebnisse:
  ##### *.txt Modellguete: Lasso-r
     file=paste("3_Ausgabe/02_Lasso-R/Modellguete.txt", sep = "")
     summary_mat[Name,] = c(round(c(R2_kal, R2_adj_kal, RMSE_kal, R2_val, R2_adj_val, RMSE_val, s.hat, N.VL), digits = 4))
   write.table(summary_mat, file = file, sep="\t", row.names = TRUE, dec = ",")
  
  #### Speichern der Kalbrierfunktion
    file = paste("3_Ausgabe/02_Lasso-R/Kalibrierfunktion_", Namensvektor[Name],".txt", sep = "")
  write.table(out, file = file, sep="\t", row.names = TRUE)
#  } 
  
  
##############################
#  HIER GEHT ES WEITER

#Anwenden der Kalbrierung
# stoffkonzentration = drop(input[1,2] + input[-1,2] %*% t(X[,wl.pos]))
